<?php
/*
*
* @name follower.php
* @package phpBB3 cBB Follower
* @version $Id: follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);
include($phpbb_root_path . 'follower/includes/functions_follower.' . $phpEx);
include($phpbb_root_path . 'includes/bbcode.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup('memberlist');

if(empty($follower->active))
{
	redirect("{$phpbb_root_path}index.$phpEx");
}

$u			= request_var('u', 0);
$mode		= request_var('mode', '');
$start		= request_var('start', 0);
$start_time	= request_var('t', time());
$final_time = $start_time;

$next_page	= (isset($_POST['next']) ? true : false);
$show_next	= true;

$wall_row = $user_cache = $comment_track_cache = $skip_modes = array();
$bbcode_bitfield = '';

$row_limit	= ($next_page) ? 10 : $config['fwr_row_limit'];

// Get readable forums
if(in_array($mode, array('home', 'mentions', 'topics', 'posts', 'search')))
{
	$forum_ids = array_keys($auth->acl_getf('f_read', true));
}

$s_hidden_fields = array('mode' => 'search');

if($_SID)
{
	$s_hidden_fields['sid'] = $_SID;
}

if(!empty($_EXTRA_URL))
{
	foreach($_EXTRA_URL as $url_param)
	{
		$url_param = explode('=', $url_param, 2);
		$s_hidden_fields[$url_param[0]] = $url_param[1];
	}
}

$template->assign_vars(array(
	'U_FWRSEARCH_ACTION'			=> $follower->u_action,
	'S_FWRSEARCH_HIDDEN_FIELDS'		=> build_hidden_fields($s_hidden_fields),
));

if($mode == 'search')
{
	include($phpbb_root_path . 'follower/includes/search.' . $phpEx);
	
	$search = new follower_search();
	$search->do_search();
	
	$template->assign_vars(array(
		'S_SHOW_HOME'			=> $follower->can_follow,
		'S_SHOW_WALL'			=> ($config['fwr_allow_comments']) ? true : false,
		'S_SHOW_NEXT'			=> (!$show_next || $row_limit > count($wall_row) || $final_time == $start_time) ? false : true,
		'S_FINAL_TIME'			=> $final_time,
		
		'U_FOLLOWER_PROFILE'	=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=follower&amp;mode=profile'),
	));

	$search->display();
}

if($u == $user->data['user_id'])
{
	$member = $user->data;
	$myself = true;
}
else
{
	$sql = 'SELECT u.*, ft.follower
		FROM ' . USERS_TABLE . ' u
		LEFT JOIN ' . FOLLOWER_TRACK_TABLE . ' ft
			ON(ft.user_id = ' . $user->data['user_id'] . " AND ft.follow_id = u.user_id AND ft.follower = 1)
		WHERE u.user_id = $u
		AND u.user_type <> " . USER_IGNORE;
	$result = $db->sql_query($sql);
	$member = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);

	$myself = false;
}

if(empty($member))
{
	trigger_error('NO_USER');
}

// a_user admins and founder are able to view inactive users and bots to be able to manage them more easily
// Normal users are able to see at least users having only changed their profile settings but not yet reactivated.
if(!$auth->acl_get('a_user') && $user->data['user_type'] != USER_FOUNDER)
{
	if($member['user_type'] == USER_IGNORE)
	{
		trigger_error('NO_USER');
	}
	else if($member['user_type'] == USER_INACTIVE && $member['user_inactive_reason'] != INACTIVE_PROFILE)
	{
		trigger_error('NO_USER');
	}
}

$user_action = $follower->u_action . 'u=' . $member['user_id'];

// Make the action buttons (tabs)
foreach($follower->actions as $action => $data)
{
	$skip = false;

	if(($myself && !$data['in_mine']) || (!$myself && !$data['in_other']))
	{
		$skip = true;
	}

	// Skip the cronology if not allow to follow
	if($action == 'home' && !$follower->can_follow)
	{
		$skip = true;
	}

	// Skip the wall tab if no comments allowed
	if($action == 'wall' && (!$config['fwr_allow_comments'] || ($myself && !$auth->acl_get('u_fwr_publish'))))
	{
		$skip = true;
	}
	
	if($skip)
	{
		$skip_modes[] = $action;
		continue;
	}

	if(!$mode)
	{
		$mode = $action;
	}

	$template->assign_block_vars('wall_action', array(
		'S_NAME'	=> $user->lang($data['lang']),
		'S_ACTIVE'	=> ($mode == $action) ? true : false,
		'U_ACTION'	=> $user_action . '&amp;mode=' . $action,
	));
}

// Redirect if no allowed and no in ajax pagination
if(in_array($mode, $skip_modes) && !$next_page)
{
	redirect($user_action);
}

switch($mode)
{
	case 'home':
		// if not allowed
		if(!$myself)
		{
			break;
		}

		$home_row = $following_users = array();
		
		$following_users = follower_get_following_users($member);

		if(empty($following_users))
		{
			break;
		}

		// Get the posts
		if(!empty($forum_ids))
		{
			$sql = 'SELECT p.post_id, p.poster_id AS user_id, p.post_subject, p.post_text AS text, p.post_time AS time, p.bbcode_uid, p.bbcode_bitfield
				FROM ' . POSTS_TABLE . ' p
					LEFT JOIN ' . TOPICS_TABLE . ' t ON(t.topic_id = p.topic_id)
				WHERE ' . $db->sql_in_set('p.poster_id', $following_users) . "
				AND p.post_approved = 1
				AND p.post_time < $start_time
				AND " . $db->sql_in_set('p.forum_id', $forum_ids) . '
					AND t.topic_first_post_id <> p.post_id
				ORDER BY p.post_time DESC';
			$result = $db->sql_query_limit($sql, $row_limit);
			while($row = $db->sql_fetchrow($result))
			{
				$home_row[$row['time']] = $row;
			}
			$db->sql_freeresult($result);
	
			// Get the topics
			$home_row += get_topic_rows($forum_ids, $following_users, $start_time);
		}

		if($config['fwr_allow_comments'])
		{
			// Get the comments (shared by following users)
			$sql = 'SELECT fc.comment_id, fc.user_id, fc.parent_id, fc.comment_text AS text, fc.comment_time AS time, ft.user_id AS share_id
				FROM ' . FOLLOWER_TRACK_TABLE . ' ft
				LEFT JOIN ' . FOLLOWER_COMMENTS_TABLE . ' fc ON(fc.comment_id = ft.comment_id)
				WHERE ' . $db->sql_in_set('ft.user_id', $following_users) . "
				AND ft.share = 1
				AND ft.track_time < $start_time
				AND ft.user_id <> " . $user->data['user_id'] . '
				ORDER BY ft.track_time DESC';
			$result = $db->sql_query_limit($sql, $row_limit);

			while($row = $db->sql_fetchrow($result))
			{
				$home_row[$row['time']] = $row;
			}
			$db->sql_freeresult($result);

			// Get the comment from the following users
			$sql = 'SELECT comment_id, user_id, parent_id, comment_text AS text, comment_time AS time
				FROM ' . FOLLOWER_COMMENTS_TABLE . '
				WHERE ' . $db->sql_in_set('user_id', $following_users) . "
				AND comment_time < $start_time
				ORDER BY comment_time DESC";
			$result = $db->sql_query_limit($sql, $row_limit);

			while($row = $db->sql_fetchrow($result))
			{
				$home_row[$row['time']] = $row;
			}
			$db->sql_freeresult($result);
		}

		if(empty($home_row))
		{
			break;
		}

		// Sort the rows
		krsort($home_row);

		$limit = min($row_limit, count($home_row));

		// Proccess
		while(sizeof($wall_row) < $limit && !empty($home_row))
		{
			$row = array_shift($home_row);

			$row_id = 0;
			$row_type = $row_text = '';

			if(isset($row['post_id']))
			{
				$row_id		= $row['post_id'];
				$row_type	= 'post';
			}
			else if(!isset($row['topic_id']))
			{
				$row_id		= $row['comment_id'];
				$row_type	= 'comment';
				$comment_track_cache[$row['comment_id']] = false;
			}

			$row_time	= (isset($row['time']) ? $row['time'] : 0);
			$is_shared	= !empty($row['share_id']);

			$wall_row[$row_time] = array(
				'id'				=> (isset($row['id']) ? $row['id'] : $row_type . '-' . $row_id),
				'user_id'			=> $row['user_id'],

				'is_share'			=> $is_shared,
				'share_id'			=> ($is_shared) ? $row['share_id'] : 0,
				'parent_id'			=> !empty($row['parent_id']),

				'post_id'			=> (isset($row['post_id']) ? $row['post_id'] : 0),
				'topic_id'			=> (isset($row['topic_id']) ? $row['topic_id'] : 0),
				'forum_id'			=> (isset($row['forum_id']) ? $row['forum_id'] : 0),
				'forum_name'		=> (isset($row['forum_name']) ? $row['forum_name'] : 0),
				'topic_replies'		=> (isset($row['topic_replies']) ? $row['topic_replies'] : 0),
				'post_subject'		=> (isset($row['post_subject']) ? $row['post_subject'] : ''),

				'text'				=> $row['text'],
				'time'				=> $row_time,

				'u_reply'			=> (isset($row['u_reply']) ? $row['u_reply'] : ''),
			);

			if(isset($row['bbcode_bitfield']))
			{
				$wall_row[$row_time] += array(
					'bbcode_uid'		=> $row['bbcode_uid'],
					'bbcode_bitfield'	=> $row['bbcode_bitfield'],
				);

				$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
			}

			$user_cache[$row['user_id']] = $row['user_id'];

			if($is_shared)
			{
				$user_cache[$row['share_id']] = $row['share_id'];
			}
		}

		unset($home_row);

	break;

	case 'wall':
		$comment_track = array();

		// Get the share comments
		$sql = 'SELECT comment_id
			FROM ' . FOLLOWER_TRACK_TABLE . '
			WHERE user_id = ' . $member['user_id'] . '
			AND share = 1';
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$comment_track[] = $row['comment_id'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT *
			FROM ' . FOLLOWER_COMMENTS_TABLE . '
			WHERE (user_id = ' . $member['user_id'] .
			(!empty($comment_track) ? ' OR ' . $db->sql_in_set('comment_id', $comment_track) : '') .
			") AND comment_time < $start_time
			ORDER BY comment_time DESC";
		$result = $db->sql_query_limit($sql, $row_limit);
		while($row = $db->sql_fetchrow($result))
		{
			$row_add = array();

			if($row['user_id'] != $member['user_id'])
			{
				$row_add = array(
					'is_share'		=> true,
					'share_id'		=> $member['user_id'],
				);

				$user_cache[$member['user_id']] = $member['user_id'];
			}

			$wall_row[] = array_merge(array(
				'id'			=> 'comment-' . $row['comment_id'],
				'user_id'		=> $row['user_id'],
				'parent_id'		=> $row['parent_id'],

				'text'			=> $row['comment_text'],
				'time'			=> $row['comment_time'],
			), $row_add);


			$user_cache[$row['user_id']] = $row['user_id'];
			$comment_track_cache[$row['comment_id']] = false;
		}

		$db->sql_freeresult($result);
	break;

	case 'mentions':
		if(!$myself && $next_page)
		{
			break;
		}

		$row_comments = $row_posts = array();

		// Get the id of items
		$sql = 'SELECT comment_id, post_id
			FROM ' . FOLLOWER_MENTIONS_TABLE . '
			WHERE user_id = ' . $member['user_id'] . "
			AND mention_time < $start_time
			ORDER BY mention_time DESC";
		$result = $db->sql_query($sql, $row_limit);
		while($row = $db->sql_fetchrow($result))
		{
			if($row['comment_id'])
			{
				$row_comments[] = $row['comment_id'];
			}
			else
			{
				$row_posts[] = $row['post_id'];
			}
		}
		$db->sql_freeresult($result);

		if($config['fwr_allow_comments'] && !empty($row_comments))
		{
			$sql = 'SELECT *
				FROM ' . FOLLOWER_COMMENTS_TABLE . '
				WHERE ' . $db->sql_in_set('comment_id', $row_comments);
			$result = $db->sql_query($sql);

			while($row = $db->sql_fetchrow($result))
			{
				$wall_row[$row['comment_time']] = array(
					'id'			=> 'comment-' . $row['comment_id'],
					'user_id'		=> $row['user_id'],
					'parent_id'		=> $row['parent_id'],

					'text'			=> $row['comment_text'],
					'time'			=> $row['comment_time'],
				);

				$user_cache[$row['user_id']] = $row['user_id'];
				$comment_track_cache[$row['comment_id']] = false;
			}

			$db->sql_freeresult($result);
		}

		if(!empty($row_posts) && !empty($forum_ids))
		{
			$sql = 'SELECT post_id, topic_id, poster_id AS user_id, post_subject, post_text AS text, post_time AS time, bbcode_uid, bbcode_bitfield
				FROM ' . POSTS_TABLE . '
				WHERE ' . $db->sql_in_set('post_id', $row_posts) . '
				AND ' . $db->sql_in_set('forum_id', $forum_ids);
			$result = $db->sql_query($sql);

			while($row = $db->sql_fetchrow($result))
			{
				$wall_row[$row['time']] = array_merge($row, array(
					'id'	=> 'post-' . $row['post_id'],
				));

				$user_cache[$row['user_id']] = $row['user_id'];

				// Define the global bbcode bitfield, will be used to load bbcodes
				$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
			}

			$db->sql_freeresult($result);
		}

		// Sort the array by time
		krsort($wall_row);
	break;

	case 'topics':
		if(empty($forum_ids))
		{
			break;
		}

		$wall_row += get_topic_rows($forum_ids, $member['user_id'], $start_time, true);
	break;

	case 'posts':
		if(empty($forum_ids))
		{
			break;
		}

		$sql = 'SELECT post_id, topic_id, post_subject, post_text AS text, post_time AS time, bbcode_uid, bbcode_bitfield
			FROM ' . POSTS_TABLE . '
			WHERE poster_id = ' . $member['user_id'] . '
			AND ' . $db->sql_in_set('forum_id', $forum_ids) . "
			AND post_time < $start_time
			ORDER BY post_time DESC";
		$result = $db->sql_query_limit($sql, $row_limit);

		while($row = $db->sql_fetchrow($result))
		{
			$wall_row[] = array_merge($row, array(
				'id'				=> 'post-' . $row['post_id'],
				'user_id'			=> $member['user_id'],
			));

			$user_cache[$member['user_id']] = $member['user_id'];

			// Define the global bbcode bitfield, will be used to load bbcodes
			$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
		}

		$db->sql_freeresult($result);
	break;

	case 'following':
		$row_count	= $member['user_following'];
		$sql_where	= 'user_id';
		$sql_select	= 'follow_id';

	// no break

	case 'followers':
		if($mode == 'followers')
		{
			$row_count	= $member['user_followers'];
			$sql_where	= 'follow_id';
			$sql_select	= 'user_id';
		}

		$sql = "SELECT $sql_select
			FROM " . FOLLOWER_TRACK_TABLE . "
			WHERE $sql_where = " . $member['user_id'] . '
			AND follower = 1';
		$result = $db->sql_query_limit($sql, $row_limit, $start);
		while($row = $db->sql_fetchrow($result))
		{
			$wall_row[$row[$sql_select]] = array(
				'id'				=> 'user-' . $row[$sql_select],
				'user_id'			=> $row[$sql_select],
			);

			$user_cache[$row[$sql_select]] = $row[$sql_select];
		}
		$db->sql_freeresult($result);

		if(!empty($wall_row))
		{
			// Check if I following this users
			$sql = 'SELECT follow_id
				FROM ' . FOLLOWER_TRACK_TABLE . '
				WHERE user_id = ' . $user->data['user_id'] . '
				AND ' . $db->sql_in_set('follow_id', $user_cache) . '
				AND follower = 1';
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$wall_row[$row['follow_id']]['is_following'] = true;
			}
			$db->sql_freeresult($result);

			$template->assign_var('PAGINATION', generate_pagination($follower->u_action . 'u=' . $member['user_id'] . "&amp;mode=$mode", $row_count, $row_limit, $start));
		}

		$show_next = false;
	break;

	// If is a invalid mode redirect and use the first mode allowed
	default:
		redirect($user_action);
	break;
}

display_follower_rows($wall_row, $next_page, $bbcode_bitfield);

$template->assign_vars(array(
	'L_MENTION'				=> $user->lang('MENTION_TITLE', $member['user_follow_name']),
	'L_NO_ITEMS'			=> ($myself) ? $user->lang['EMPTY_MY_' . strtoupper($mode)] : $user->lang('EMPTY_OTHER_' . strtoupper($mode), $member['username']),
	'S_MYSELF'				=> $myself,

	'S_USER_FULLNAME'		=> $member['username'],
	'S_USER_FOLLOWNAME'		=> $member['user_follow_name'],
	'S_USER_DESCRIPTION'	=> bbcode_nl2br($member['user_aboutme']),
	'S_USER_AVATAR'			=> ($user->optionget('viewavatars')) ? get_user_avatar($member['user_avatar'], $member['user_avatar_type'], $member['user_avatar_width'], $member['user_avatar_height']) : '', 

	'S_SHOW_HOME'			=> $follower->can_follow,
	'S_SHOW_WALL'			=> (in_array('wall', $skip_modes) ? false : true),

	'S_FOLLOW_BUTTON'		=> (!$follower->can_follow || $myself) ? '' : $follower->buttons->create('follow_button', array(
		'USER_ID'		=> $member['user_id'],
		'USER_STATUS'	=> (!empty($member['follower']) ? 'following' : 'follow')
	)),
	
	'S_MENTION_BUTTON'	=> $follower->buttons->create('mention', array(
		'FOLLOWNAME'	=> $member['user_follow_name'],
		'TITLE'			=> $user->lang('MENTION_TITLE', $member['user_follow_name']),
		'STYLE'			=> 'style="float:right;"',
	)),

	'S_SHOW_NEXT'			=> (!$show_next || $row_limit > count($wall_row) || $final_time == $start_time) ? false : true,
	'S_FINAL_TIME'			=> $final_time,

	'U_FOLLOWER_PROFILE'	=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=follower&amp;mode=profile'),
));

if($myself)
{
	follower_get_who_follow($member);
	follower_get_trends();
}

$template->set_filenames(array(
	'body' => 'follower/follower_body.html'
));

$page_title = ($myself) ? $user->lang[strtoupper($mode)] : $user->lang('TITLE_' . strtoupper($mode), get_followname_string('label', $member['user_follow_name']));
page_header($page_title);

if($next_page)
{
	$template->assign_var('S_READMODE', true);
}
else
{
	load_user_information();
}

page_footer();


// Get the topics
function get_topic_rows($forum_ids, $user_ids, $start_time, $grab_user = false)
{
	global $db, $auth, $user_cache, $row_limit, $phpbb_root_path, $phpEx;

	$result_ary = array();

	if(!is_array($user_ids))
	{
		$user_ids = array($user_ids);
	}

	$sql = 'SELECT f.forum_name, t.forum_id, t.topic_id, t.topic_title AS text, t.topic_replies, t.topic_time AS time, t.topic_poster AS user_id
		FROM ' . TOPICS_TABLE . ' t
		LEFT JOIN ' . FORUMS_TABLE . ' f ON(f.forum_id = t.forum_id)
		WHERE ' . $db->sql_in_set('t.topic_poster', $user_ids) . '
		AND t.topic_approved = 1
		AND t.topic_status <> ' . ITEM_MOVED . '
		AND ' . $db->sql_in_set('t.forum_id', $forum_ids) . "
		AND t.topic_time < $start_time
		ORDER BY t.topic_time DESC";
	$result = $db->sql_query_limit($sql, $row_limit);
	while($row = $db->sql_fetchrow($result))
	{
		$result_ary[$row['time']] = array_merge($row, array(
			'id'			=> 'topic-' . $row['topic_id'],
			'u_reply'		=> ($auth->acl_get('f_reply', $row['forum_id']) ? append_sid("{$phpbb_root_path}posting.$phpEx", 'mode=reply&amp;f=' . $row['forum_id'] . '&amp;t=' . $row['topic_id']) : ''),
		));

		if($grab_user)
		{
			$user_cache[$row['user_id']] = $row['user_id'];
		}
	}
	$db->sql_freeresult($result);

	return $result_ary;
}

/**
* Prepare profile data
*/
function show_wall_profile($data, $user_notes_enabled = false, $warn_user_enabled = false)
{
	global $config, $auth, $template, $user, $phpEx, $phpbb_root_path;

	$username	= $data['username'];
	$user_id	= $data['user_id'];

	$rank_title = $rank_img = $rank_img_src = '';
	$last_visit = $email = $age = '';
	$online = false;

	get_user_rank($data['user_rank'], (($user_id == ANONYMOUS) ? false : $data['user_posts']), $rank_title, $rank_img, $rank_img_src);

	if ((!empty($data['user_allow_viewemail']) && $auth->acl_get('u_sendemail')) || $auth->acl_get('a_user'))
	{
		$email = ($config['board_email_form'] && $config['email_enable']) ? append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=email&amp;u=' . $user_id) : (($config['board_hide_emails'] && !$auth->acl_get('a_user')) ? '' : 'mailto:' . $data['user_email']);
	}

	if ($config['load_onlinetrack'])
	{
		$update_time = $config['load_online_time'] * 60;
		$online = (time() - $update_time < $data['session_time'] && ((isset($data['session_viewonline']) && $data['session_viewonline']) || $auth->acl_get('u_viewonline'))) ? true : false;
	}

	if ($data['user_allow_viewonline'] || $auth->acl_get('u_viewonline'))
	{
		$last_visit = (!empty($data['session_time'])) ? $data['session_time'] : $data['user_lastvisit'];
	}

	if ($config['allow_birthdays'] && $data['user_birthday'])
	{
		list($bday_day, $bday_month, $bday_year) = array_map('intval', explode('-', $data['user_birthday']));

		if ($bday_year)
		{
			$now = getdate(time() + $user->timezone + $user->dst - date('Z'));

			$diff = $now['mon'] - $bday_month;
			if ($diff == 0)
			{
				$diff = ($now['mday'] - $bday_day < 0) ? 1 : 0;
			}
			else
			{
				$diff = ($diff < 0) ? 1 : 0;
			}

			$age = (int) ($now['year'] - $bday_year - $diff);
		}
	}

	// Dump it out to the template
	$template->assign_vars(array(
		'AGE'				=> $age,
		'ICQ_STATUS_IMG'	=> (!empty($data['user_icq'])) ? '<img src="http://web.icq.com/whitepages/online?icq=' . $data['user_icq'] . '&amp;img=5" width="18" height="18" />' : '',
		'JOINED'			=> $user->format_date($data['user_regdate']),
		'LOCATION'			=> ($data['user_from']) ? $data['user_from'] : '',
		'POSTS'				=> ($data['user_posts']) ? $data['user_posts'] : 0,
		'RANK_IMG'			=> $rank_img,
		'RANK_IMG_SRC'		=> $rank_img_src,
		'RANK_TITLE'		=> $rank_title,
		'USERNAME'			=> $data['username'],
		'VISITED'			=> (empty($last_visit)) ? ' - ' : $user->format_date($last_visit),
		'WARNINGS'			=> isset($data['user_warnings']) ? $data['user_warnings'] : 0,

		'S_JABBER_ENABLED'	=> ($config['jab_enable']) ? true : false,
		'S_ONLINE'			=> ($config['load_onlinetrack'] && $online) ? true : false,
		'S_WARNINGS'		=> ($auth->acl_getf_global('m_') || $auth->acl_get('m_warn')) ? true : false,

		'USER_ICQ'			=> $data['user_icq'],
		'USER_AIM'			=> $data['user_aim'],
		'USER_YIM'			=> $data['user_yim'],
		'USER_MSN'			=> $data['user_msnm'],
		'USER_JABBER'		=> $data['user_jabber'],
		'USER_JABBER_IMG'	=> ($data['user_jabber']) ? $user->img('icon_contact_jabber', $data['user_jabber']) : '',
		
		'U_SEARCH_USER'	=> ($auth->acl_get('u_search')) ? append_sid("{$phpbb_root_path}search.$phpEx", "author_id=$user_id&amp;sr=posts") : '',
		'U_NOTES'		=> ($user_notes_enabled && $auth->acl_getf_global('m_')) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=notes&amp;mode=user_notes&amp;u=' . $user_id, true, $user->session_id) : '',
		'U_WARN'		=> ($warn_user_enabled && $auth->acl_get('m_warn')) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=warn&amp;mode=warn_user&amp;u=' . $user_id, true, $user->session_id) : '',
		'U_PM'			=> ($config['allow_privmsg'] && $auth->acl_get('u_sendpm') && ($data['user_allow_pm'] || $auth->acl_gets('a_', 'm_') || $auth->acl_getf_global('m_'))) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=pm&amp;mode=compose&amp;u=' . $user_id) : '',
		'U_EMAIL'		=> $email,
		'U_WWW'			=> (!empty($data['user_website'])) ? $data['user_website'] : '',
		'U_SHORT_WWW'	=> (!empty($data['user_website'])) ? ((strlen($data['user_website']) > 55) ? substr($data['user_website'], 0, 39) . ' ... ' . substr($data['user_website'], -10) : $data['user_website']) : '',
		'U_ICQ'			=> ($data['user_icq']) ? 'http://www.icq.com/people/webmsg.php?to=' . urlencode($data['user_icq']) : '',
		'U_AIM'			=> ($data['user_aim'] && $auth->acl_get('u_sendim')) ? append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=contact&amp;action=aim&amp;u=' . $user_id) : '',
		'U_YIM'			=> ($data['user_yim']) ? 'http://edit.yahoo.com/config/send_webmesg?.target=' . urlencode($data['user_yim']) . '&amp;.src=pg' : '',
		'U_MSN'			=> ($data['user_msnm'] && $auth->acl_get('u_sendim')) ? append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=contact&amp;action=msnm&amp;u=' . $user_id) : '',
		'U_JABBER'		=> ($data['user_jabber'] && $auth->acl_get('u_sendim')) ? append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=contact&amp;action=jabber&amp;u=' . $user_id) : '',
	));
}

function load_user_information()
{
	global $db, $user, $template, $config, $auth, $phpbb_root_path, $phpEx;
	global $member, $myself, $user_action;
	
	$zebra_enabled = $friends_enabled = $foes_enabled = $user_notes_enabled = $warn_user_enabled = false;
	$profile_fields = array();

	// What colour is the zebra
	$sql = 'SELECT friend, foe
		FROM ' . ZEBRA_TABLE . '
		WHERE zebra_id = ' . $member['user_id'] . "
			AND user_id = {$user->data['user_id']}";

	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	$foe = ($row['foe']) ? true : false;
	$friend = ($row['friend']) ? true : false;
	$db->sql_freeresult($result);

	if ($config['load_onlinetrack'])
	{
		$sql = 'SELECT MAX(session_time) AS session_time, MIN(session_viewonline) AS session_viewonline
			FROM ' . SESSIONS_TABLE . '
			WHERE session_user_id = ' . $member['user_id'];
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$member['session_time'] = (isset($row['session_time'])) ? $row['session_time'] : 0;
		$member['session_viewonline'] = (isset($row['session_viewonline'])) ? $row['session_viewonline'] :	0;
		unset($row);
	}

	// Only check if the user is logged in
	if ($user->data['is_registered'])
	{
		if (!class_exists('p_master'))
		{
			include($phpbb_root_path . 'includes/functions_module.' . $phpEx);
		}
		$module = new p_master();

		$module->list_modules('ucp');
		$module->list_modules('mcp');

		$user_notes_enabled	= ($module->loaded('notes', 'user_notes')) ? true : false;
		$warn_user_enabled 	= ($module->loaded('warn', 'warn_user')) ? true : false;
		$zebra_enabled 		= ($module->loaded('zebra')) ? true : false;
		$friends_enabled 	= ($module->loaded('zebra', 'friends')) ? true : false;
		$foes_enabled 		= ($module->loaded('zebra', 'foes')) ? true : false;

		unset($module);
	}

	show_wall_profile($member, $user_notes_enabled, $warn_user_enabled);

	// Custom Profile Fields
	if ($config['load_cpf_viewprofile'])
	{
		if(!class_exists('custom_profile'))
		{
			include($phpbb_root_path . 'includes/functions_profile_fields.' . $phpEx);
		}

		$cp = new custom_profile();
		$profile_fields = $cp->generate_profile_fields_template('grab', $member['user_id']);
		$profile_fields = (isset($profile_fields[$member['user_id']])) ? $cp->generate_profile_fields_template('show', false, $profile_fields[$member['user_id']]) : array();
	}

	if($config['fwr_default'])
	{
		// Do the relevant calculations
		$memberdays		= max(1, round((time() - $member['user_regdate']) / 86400));
		$posts_per_day	= $member['user_posts'] / $memberdays;
		$percentage		= ($config['num_posts']) ? min(100, ($member['user_posts'] / $config['num_posts']) * 100) : 0;

		if($config['load_user_activity'])
		{
			display_user_activity($member);
		}

		// If the user has m_approve permission or a_user permission, then list then display unapproved posts
		$member['posts_in_queue'] = 0;

		if($auth->acl_getf_global('m_approve') || $auth->acl_get('a_user'))
		{
			$sql = 'SELECT COUNT(post_id) as posts_in_queue
				FROM ' . POSTS_TABLE . '
				WHERE poster_id = ' . $member['user_id'] . '
					AND post_approved = 0';
			$result = $db->sql_query($sql);
			$member['posts_in_queue'] = (int) $db->sql_fetchfield('posts_in_queue');
			$db->sql_freeresult($result);
		}

		$template->assign_vars(array(
			'L_POSTS_IN_QUEUE'		=> $user->lang('NUM_POSTS_IN_QUEUE', $member['posts_in_queue']),
			'POSTS_IN_QUEUE'		=> $member['posts_in_queue'],

			'POSTS_DAY'				=> sprintf($user->lang['POST_DAY'], $posts_per_day),
			'POSTS_PCT'				=> sprintf($user->lang['POST_PCT'], $percentage),
			'U_USER_ADMIN'			=> ($auth->acl_get('a_user')) ? append_sid("{$phpbb_root_path}adm/index.$phpEx", 'i=users&amp;mode=overview&amp;u=' . $member['user_id'], true, $user->session_id) : '',
			'U_USER_BAN'			=> ($auth->acl_get('m_ban') && !$myself) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=ban&amp;mode=user&amp;u=' . $member['user_id'], true, $user->session_id) : '',
			'U_MCP_QUEUE'			=> ($auth->acl_getf_global('m_approve')) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=queue', true, $user->session_id) : '',
			'U_SWITCH_PERMISSIONS'	=> ($auth->acl_get('a_switchperm') && !$myself) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=switch_perm&amp;u=' . $member['user_id'] . '&amp;hash=' . generate_link_hash('switchperm')) : '',
		));
	}

	if(!empty($profile_fields['blockrow']))
	{
		foreach ($profile_fields['blockrow'] as $field_data)
		{
			$template->assign_block_vars('custom_fields', $field_data);
		}
	}

	// Inactive reason/account?
	if($member['user_type'] == USER_INACTIVE)
	{
		$user->add_lang('acp/common');

		$inactive_reason = $user->lang['INACTIVE_REASON_UNKNOWN'];

		switch ($member['user_inactive_reason'])
		{
			case INACTIVE_REGISTER:
				$inactive_reason = $user->lang['INACTIVE_REASON_REGISTER'];
			break;

			case INACTIVE_PROFILE:
				$inactive_reason = $user->lang['INACTIVE_REASON_PROFILE'];
			break;

			case INACTIVE_MANUAL:
				$inactive_reason = $user->lang['INACTIVE_REASON_MANUAL'];
			break;

			case INACTIVE_REMIND:
				$inactive_reason = $user->lang['INACTIVE_REASON_REMIND'];
			break;
		}

		$template->assign_vars(array(
			'S_USER_INACTIVE'		=> true,
			'USER_INACTIVE_REASON'	=> $inactive_reason,
		));
	}

	$template->assign_vars(array(
		'SHOW_USER_INFORMATION'	=> true,
		'S_NOTE_COUNT'			=> $member['user_comments'],
		'S_FOLLOWING_COUNT'		=> $member['user_following'],
		'S_FOLLOWERS_COUNT'		=> $member['user_followers'],

		'L_INFORMATION_TITLE'	=> $user->lang('USER_INFO', get_followname_string('label', $member['user_follow_name'])),

		'U_NOTES'				=> "$user_action&amp;mode=wall",
		'U_FOLLOWING'			=> "$user_action&amp;mode=following",
		'U_FOLLOWERS'			=> "$user_action&amp;mode=followers",

		'OCCUPATION'	=> (!empty($member['user_occ'])) ? censor_text($member['user_occ']) : '',
		'INTERESTS'		=> (!empty($member['user_interests'])) ? censor_text($member['user_interests']) : '',

		'PM_IMG'		=> $user->img('icon_contact_pm', $user->lang['SEND_PRIVATE_MESSAGE']),
		'EMAIL_IMG'		=> $user->img('icon_contact_email', $user->lang['EMAIL']),
		'WWW_IMG'		=> $user->img('icon_contact_www', $user->lang['WWW']),
		'ICQ_IMG'		=> $user->img('icon_contact_icq', $user->lang['ICQ']),
		'AIM_IMG'		=> $user->img('icon_contact_aim', $user->lang['AIM']),
		'MSN_IMG'		=> $user->img('icon_contact_msnm', $user->lang['MSNM']),
		'YIM_IMG'		=> $user->img('icon_contact_yahoo', $user->lang['YIM']),
		'JABBER_IMG'	=> $user->img('icon_contact_jabber', $user->lang['JABBER']),
		'SEARCH_IMG'	=> $user->img('icon_user_search', $user->lang['SEARCH']),

		'S_USER_NOTES'		=> ($user_notes_enabled) ? true : false,
		'S_WARN_USER'		=> ($warn_user_enabled) ? true : false,
		'S_ZEBRA'			=> (!$myself && $user->data['is_registered'] && $zebra_enabled) ? true : false,
		'U_ADD_FRIEND'		=> (!$friend && !$foe && $friends_enabled) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=zebra&amp;add=' . urlencode(htmlspecialchars_decode($member['username']))) : '',
		'U_ADD_FOE'			=> (!$friend && !$foe && $foes_enabled) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=zebra&amp;mode=foes&amp;add=' . urlencode(htmlspecialchars_decode($member['username']))) : '',
		'U_REMOVE_FRIEND'	=> ($friend && $friends_enabled) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=zebra&amp;remove=1&amp;usernames[]=' . $member['user_id']) : '',
		'U_REMOVE_FOE'		=> ($foe && $foes_enabled) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=zebra&amp;remove=1&amp;mode=foes&amp;usernames[]=' . $member['user_id']) : '',
	));
}

?>
