<?php
/*
*
* @name follower.php
* @package phpBB3 cBB Follower
* @version $Id: follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class follower
{
	var $actions = array(
		'home'		=> array('lang' => 'HOME',		'in_mine' => true,	'in_other' => false),
		'wall'		=> array('lang' => 'WALL',		'in_mine' => true,	'in_other' => true),
		'mentions'	=> array('lang' => 'MENTIONS',	'in_mine' => true,	'in_other' => false),
		'topics'	=> array('lang' => 'TOPICS',	'in_mine' => true,	'in_other' => true),
		'posts'		=> array('lang' => 'POSTS',		'in_mine' => true,	'in_other' => true),
		'following'	=> array('lang' => 'FOLLOWING',	'in_mine' => true,	'in_other' => true),
		'followers'	=> array('lang' => 'FOLLOWERS',	'in_mine' => true,	'in_other' => true),
	);

	var $row_actions = array(
		'reply'		=> array('lang' => 'REPLY',		'is_mine' => true,	'is_other' => true,		'is_share' => true,		'u_' => 'u_fwr_share'),
		'share'		=> array('lang' => 'SHARE',		'is_mine' => false,	'is_other' => true,		'is_share' => true,		'u_' => 'u_fwr_share'),
		'unshare'	=> array('lang' => 'UNSHARE',	'is_mine' => false,	'is_other' => false,	'is_share' => true,		'u_' => 'u_fwr_share'),
		'delete'	=> array('lang' => 'DELETE',	'is_mine' => true,	'is_other' => true,		'is_share' => false,	'u_' => 'u_fwr_delete',	'm_' => 'm_fwr_delete'),
	);

	// Keep one space between values (pm and email)
	var $key_notify_options	= array('replies' => 0, 'mentions' => 2, 'follower' => 4, 'share' => 6, 'topics' => 8, 'posts' => 10);
	var $key_notify_values	= array();

	var $u_action;
	var $active			= false;
	var $can_follow		= false;
	var $default_profile	= false;
	var $hashtags		= false;
	var $installed		= false;
	var $mention_ids	= false;
	var $mention_users 	= array();
	var $buttons;

	function follower()
	{
		global $config, $phpbb_root_path, $phpEx, $table_prefix;

		if(!empty($config['fwr_version']))
		{
			$this->installed = true;
		}

		if($this->installed && !defined('IN_INSTALL'))
		{
			$this->active	= (!empty($config['fwr_enabled']) ? true : false);
			include($phpbb_root_path . 'follower/includes/constants.' . $phpEx);
		}
	}

	function setup()
	{
		global $phpbb_root_path, $db, $user, $auth, $config, $template, $phpEx;
		
		// Deactivate if the user is anonymous or a bot, and in the acp
		if(empty($user->data['is_registered']) || defined('ADMIN_START'))
		{
			$this->active = false;	
		}

		if(!$this->active)
		{
			return false;
		}
		
		include($phpbb_root_path . 'follower/includes/buttons.' . $phpEx);
		$this->buttons = new fwr_buttons();
	
		$this->default_profile	= ($config['fwr_default']) ? true : false;

		$user->add_lang('mods/follower/common');

		// Reset following users if not allow to follow
		if(!$auth->acl_get('u_fwr_follow') && $user->data['user_following'] > 0)
		{
			$sql = 'DELETE FROM ' . FOLLOWER_TRACK_TABLE . '
				WHERE user_id = ' . $user->data['user_id'] . '
				AND follower = 1';
			$db->sql_query($sql);

			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_following = 0
				WHERE user_id = ' . $user->data['user_id'];
			$db->sql_query($sql);

			$user->data['user_following'] = 0;
		}

		$this->u_action = append_sid("{$phpbb_root_path}follower.$phpEx");
		$this->u_action .= ((strpos($this->u_action, '?') === false) ? '?' : '&amp;');

		$this->can_follow = ($auth->acl_get('u_fwr_follow')) ? true : false;

		$template->assign_vars(array(
			'FWR_ENABLED'			=> true,
			'FWR_VERSION'			=> $config['fwr_version'],
			'FWR_DEFAULT'			=> $this->default_profile,
			'FWR_MAXLENGTH'			=> $config['fwr_maxlength'],
			'FWR_ALLOW_COMMENTS'	=> ($config['fwr_allow_comments'] && $auth->acl_get('u_fwr_publish')) ? true : false,
			'FWR_CAN_MENTION'		=> ($auth->acl_get('u_fwr_mention')) ? true : false,
			
			'S_SLIDER_AUTOHIDE'		=> ($config['fwr_slider_autohide']) ? true : false,
			'S_SLIDER_POS'			=> $config['fwr_slider_pos'],
			
			'S_NEW_NOTIFY'		=> (!empty($user->data['user_new_notify']) ? $user->data['user_new_notify'] : 0),
			'T_FOLLOWER_PATH'	=> generate_board_url() . '/follower/',

			'U_FOLLOWER_HOME'	=> $this->u_action . 'u=' . $user->data['user_id'] . '&amp;mode=home',
			'U_FOLLOWER_WALL'	=> $this->u_action . 'u=' . $user->data['user_id'] . '&amp;mode=wall',
			'U_FOLLOWER_CORE'	=> append_sid($phpbb_root_path . $user->page['page'], false, false),
		));

		// If it is a ajax request, do the action here and exit
		$do_core = request_var('core', 1);

		if(isset($_POST['jfcore']) && $do_core)
		{
			include($phpbb_root_path . 'follower/includes/functions_follower_core.' . $phpEx);
			do_core_action();
			exit_handler();
		}
	}
	
	function parse(&$text)
	{
		$this->_parse_mentions($text);
		$this->_parse_hashtags($text);
	}

	function _parse_mentions(&$text)
	{
		global $db, $auth;
		
		$this->mention_ids = array();
		
		if(!$this->active || !$auth->acl_get('u_fwr_mention'))
		{
			return false;
		}

		// First put all mention follow names into lowercase
		$text = preg_replace('#((?:^|\s)+@)([^<>\s]+)#e', "'\$1' . strtolower('\$2')", $text);

		preg_match_all('#(?:^|\s)+@([^\[<>\s]+)#', $text, $matches1);
		preg_match_all('#(?:^|\s)+@\[([^<>\[\]]+)\]#', $text, $matches2);
		
		$sql_where = '';

		if(!empty($matches1[1]))
		{
			$sql_where .= 'WHERE ' . $db->sql_in_set('LOWER(user_follow_name)', array_unique($matches1[1]));
		}
		
		if(!empty($matches2[1]))
		{
			$matches2[1] = array_unique($matches2[1]);

			foreach($matches2[1] as $id => $mtext)
			{
				$matches2[1][$id] = utf8_clean_string($mtext);
			}
			
			$sql_where .= (($sql_where) ? ' OR ' : 'WHERE ') . $db->sql_in_set('username_clean', $matches2[1]);
		}

		// Get the users ids
		if($sql_where)
		{
			$sql = 'SELECT user_id, username, username_clean, user_follow_name
				FROM ' . USERS_TABLE . "
				$sql_where
				AND user_type <> " . USER_IGNORE;
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$find = array(
					'@' . strtolower($row['user_follow_name']),
					'@[' . $row['username'] . ']',
					'@[' . $row['username_clean'] . ']',
				);
				
				$text = str_replace($find, '<!--' . $row['user_id'] . ':@' . $row['user_follow_name'] . '-->', $text);

				$this->mention_ids[] = $row['user_id'];
			}
			$db->sql_freeresult($result);
		}
	}
	
	function _parse_hashtags(&$text)
	{		
		$this->hashtags = array();
		
		if(!$this->active)
		{
			return false;
		}

		preg_match_all('/(?:^|\s)+(#[^<>\s]+)/', $text, $matches);

		if(!empty($matches[1]))
		{
			foreach($matches[1] as $tag)
			{
				$this->hashtags[] = str_replace('#', '', strtolower($tag));
				$text = str_replace($tag, '<!--' . $tag . '-->', $text);
			}
			
			$this->hashtags = array_unique($this->hashtags);
		}
	}
	
	function store($post_id = 0, $comment_id = 0, $time = 0)
	{
		$this->_store_mentions($post_id, $comment_id, $time);
		$this->_store_hashtags($post_id, $comment_id);
	}

	function _store_mentions($post_id, $comment_id, $time)
	{
		global $db;

		if($this->mention_ids !== false && ($post_id || $comment_id) && $time)
		{
			$sql_ary = array();

			// First, delete all mentions on the post/comment
			$sql = 'DELETE FROM ' . FOLLOWER_MENTIONS_TABLE . "
				WHERE post_id = $post_id
				AND comment_id = $comment_id";
			$db->sql_query($sql);

			foreach($this->mention_ids as $user_id)
			{
				$sql_ary[] = array(
					'user_id'		=> $user_id,
					'post_id'		=> $post_id,
					'comment_id'	=> $comment_id,
					'mention_time'	=> $time,
				);
			}

			if(!empty($sql_ary))
			{
				$db->sql_multi_insert(FOLLOWER_MENTIONS_TABLE, $sql_ary);
			}

			unset($sql_ary);
		}
	}
	
	function _store_hashtags($post_id, $comment_id)
	{
		global $db;

		if($this->hashtags !== false && ($post_id || $comment_id))
		{
			$sql_ary = array();

			// First, delete all hashtags on the post/comment
			$sql = 'DELETE FROM ' . FOLLOWER_HASHTAGS_TABLE . "
				WHERE post_id = $post_id
				AND comment_id = $comment_id";
			$db->sql_query($sql);

			foreach($this->hashtags as $hashtag)
			{
				$sql_ary[] = array(
					'hashtag'		=> $hashtag,
					'post_id'		=> $post_id,
					'comment_id'	=> $comment_id,
				);
			}

			if(!empty($sql_ary))
			{
				$db->sql_multi_insert(FOLLOWER_HASHTAGS_TABLE, $sql_ary);
			}

			unset($sql_ary);
		}
	}

	function display(&$text)
	{
		// Display Mentions
		$text = preg_replace('#<!\-\-([0-9]+):(@[^<>\s]+)\-\->#e', "get_followname_string('full', '\\2', \\1)", $text);
	
		// Display HashTags
		$text = preg_replace('/<!\-\-#([^<>\s]+)\-\->/e', "get_hashtag_string('\\1')", $text);
	}

	function edit(&$text)
	{
		// mentions
		$text = preg_replace('#<!\-\-[0-9]+:(@[^<>\s]+)\-\->#', '\\1', $text);
		
		// hashtags
		$text = preg_replace('/<!\-\-(#[^<>\s]+)\-\->/', '\\1', $text);
	}

	/**
	* Get or Set notify bit field from user options
	*/
	function notify($key, $type, $data = false, $value = false)
	{
		global $user;

		if(!$this->active)
		{
			return false;
		}

		$var = ($data) ? $data : $user->data['user_notify_options'];
		$num = ($type == 'pm') ? 0 : 1;

		// Get
		if($value === false)
		{
			if (!isset($this->key_notify_values[$key]))
			{
				$this->key_notify_values[$key] = array(
					'pm'	=> ($var & 1 << $this->key_notify_options[$key]) ? true : false,
					'email' => ($var & 1 << $this->key_notify_options[$key] + 1) ? true : false,
				);
			}

			return $this->key_notify_values[$key][$type];
		}
		// Set
		else
		{
			if ($value && !($var & 1 << $this->key_notify_options[$key] + $num))
			{
				$var += 1 << $this->key_notify_options[$key] + $num;
			}
			else if (!$value && ($var & 1 << $this->key_notify_options[$key] + $num))
			{
				$var -= 1 << $this->key_notify_options[$key] + $num;
			}
			else
			{
				return ($data) ? $var : false;
			}

			if (!$data)
			{
				$user->data['user_notify_options'] = $var;
				return true;
			}
			else
			{
				return $var;
			}
		}
	}
}

function get_followname_string($mode, $follow_name, $user_id = 0)
{
	global $follower;
	
	if(is_array($follow_name))
	{
		$user_id		= $follow_name['user_id'];
		$follow_name	= $follow_name['user_follow_name'];
	}

	switch($mode)
	{
		case 'name_active':
			if(!$follower->active)
			{
				return '';
			}

		case 'label':
		case 'name':
		case 'full':
			if(strpos($follow_name, '@') !== 0)
			{
				$follow_name = '@' . $follow_name;
			}
			
			if($mode == 'label' || !$follower->active)
			{
				return $follow_name;	
			}

			if($mode == 'name_active' || $mode == 'name')
			{
				return '<span class="js-follower-name">' . $follow_name . '</span>';
			}
			
		// no break
			
		case 'profile':
			$profile_url = $follower->u_action . 'u=' . $user_id . '&amp;mode=wall';
			
			if($mode == 'profile')
			{
				return $profile_url;
			}
			
			return '<a class="fw-mention-link" href="' . $profile_url . '"><strong>' . $follow_name . '</strong></a>';
		break;
		
		default:
			return '';
		break;
	}
}

function get_hashtag_string($hashtag)
{
	global $follower;
	
	if(!$follower->active)
	{
		return '#' . $hashtag;
	}

	return '<a class="fw-hashtag-link" href="' . $follower->u_action . 'mode=search&amp;type=hashtag&amp;keywords=' . urlencode(strtolower($hashtag)) . '"><strong>#' . $hashtag . '</strong></a>';
}

?>
