<?php
/*
*
* @name functions_follower.php
* @package phpBB3 cBB Follower
* @version $Id: functions_follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

function display_follower_rows($wall_row, $next_page = false, $bbcode_bitfield = '', $main_comment = false)
{
	global $db, $user, $auth, $template, $phpbb_root_path, $phpEx;
	global $follower, $user_cache, $comment_track_cache, $final_time;

	if(!sizeof($wall_row) || empty($user_cache))
	{
		return;
	}

	// Get the user information
	$sql = 'SELECT user_id, username, user_follow_name, user_aboutme, 
		user_avatar, user_avatar_type, user_avatar_width, user_avatar_height
		FROM ' . USERS_TABLE . '
		WHERE ' . $db->sql_in_set('user_id', $user_cache);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$user_cache[$row['user_id']] = $row;
	}
	$db->sql_freeresult($result);

	// Is the comments shared for me?
	if(sizeof($comment_track_cache) && $auth->acl_get('u_fwr_share'))
	{
		$sql = 'SELECT comment_id
			FROM ' . FOLLOWER_TRACK_TABLE . '
			WHERE user_id = ' . $user->data['user_id'] . '
			AND share = 1
			AND ' . $db->sql_in_set('comment_id', array_keys($comment_track_cache));
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$comment_track_cache[$row['comment_id']] = true;
		}
		$db->sql_freeresult($result);
	}

	// Instantiate BBCode if need be
	if($bbcode_bitfield !== '')
	{
		$bbcode = new bbcode(base64_encode($bbcode_bitfield));
	}

	foreach($wall_row as $id => $row)
	{
		$s_actions = '';

		list($row_type, $row_id) = explode('-', $row['id'], 2);
		$row_id = (int)	$row_id;

		$is_mine		= ($row['user_id'] == $user->data['user_id']) ? true : false;
		$is_shared		= !empty($row['is_share']);
		$is_my_share	= ($row_type == 'comment' && !empty($comment_track_cache[$row_id])) ? true : false;

		// Add the user information
		$row += $user_cache[$row['user_id']];

		if($row_type == 'user')
		{
			$row['text'] = $row['user_aboutme'];
		}

		if($row_type != 'user')
		{
			foreach($follower->row_actions as $action => $data)
			{
				if($row_type == 'comment')
				{
					$row["u_$action"] = "#!$action-comment"; 
				}

				if(empty($row["u_$action"]) || ($is_mine && !$data['is_mine']) ||
					(!$is_mine && !$data['is_other'] && !$is_my_share) ||
					($is_my_share && !$data['is_share']) || ($row_type == 'comment' && !$auth->acl_get($data['u_'])))
				{
					continue;
				}
				
				// I can moderate the comments from other users?
				if($action == 'delete' && !$is_mine && !$auth->acl_get($data['m_']))
				{
					continue;
				}

				if($action == 'share' && ($row_type != 'comment' || $is_my_share))
				{
					continue;
				}

				if($action == 'unshare' && ($row_type != 'comment' || !$is_my_share))
				{
					continue;
				}

				$s_actions .= '<span class="wall-row-icon icon-' . $action . '"></span>';
				$s_actions .= '<a href="' . $row["u_$action"] . '" rel="#' . $action . '">';
				$s_actions .= $user->lang[$data['lang']] . '</a>&nbsp;&nbsp;';
			}
		}

		if($row_type != 'topic')
		{
			// Parse the message and subject
			$message = censor_text($row['text']);

			// Second parse bbcode here
			if (!empty($row['bbcode_bitfield']))
			{
				follower_bbcode_strip($message, $row['bbcode_uid']);
				$bbcode->bbcode_second_pass($message, $row['bbcode_uid'], $row['bbcode_bitfield']);
			}

			$message = bbcode_nl2br($message);
			$message = smiley_text($message);
			
			// Parse urls
			$message = make_clickable($message, false, 'fw-row-link');
		}

		if(!empty($row['topic_id']))
		{
			$u_viewtopic = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 't=' . $row['topic_id']);
		}

		if($row_type == 'post')
		{
			$u_viewpost	= append_sid("{$phpbb_root_path}viewtopic.$phpEx", 't=' . $row['topic_id'] . '&amp;p=' . $row['post_id'] . '#p' . $row['post_id']);
			$message	= sprintf($user->lang['POST_ROW'], $u_viewpost, $row['post_subject'], html_truncate($message, 200, '...'));
		}
		else if($row_type == 'topic')
		{
			$u_viewforum	= append_sid("{$phpbb_root_path}viewforum.$phpEx", 'f=' . $row['forum_id']);
			$message		= sprintf($user->lang['TOPIC_ROW'], $u_viewtopic, $row['text'], $row['topic_replies'], $u_viewforum, $row['forum_name']);
		}

		$template->assign_block_vars('wallrow', array(
			'S_MAIN_COMMENT'	=> ($row_type == 'comment' && $main_comment == $row_id) ? true : false,

			'ID'				=> $row_id,
			'ROW_ID'			=> $row['id'],
			'USER_ID'			=> ($user->data['user_type'] != USER_IGNORE && $row_type == 'user') ? $row_id : '',

			'IS_MINE'			=> $is_mine,
			'IS_SHARE'			=> $is_shared,
			'IS_MY_SHARE'		=> $is_my_share,
			'IS_COMMENT'		=> ($row_type == 'comment') ? true : false,
			'IS_REPLY'			=> !empty($row['parent_id']),
			'S_SHARE_INFO'		=> (($is_shared && $row['share_id'] != $user->data['user_id']) ? $user->lang('SHARED_BY', $user_cache[$row['share_id']]['user_follow_name']) : ''),

			'S_AVATAR'			=> ($user->optionget('viewavatars')) ? get_user_avatar($row['user_avatar'], $row['user_avatar_type'], $row['user_avatar_width'], $row['user_avatar_height']) : '',
			'S_FOLLOWNAME'		=> $row['user_follow_name'],
			'S_FULLNAME'		=> $row['username'],
			'S_CONTENT'			=> $message,
			'S_TIME_MARK'		=> ($next_page && !empty($row['time']) ? $row['time'] : 0),
			'S_TIME'			=> (!empty($row['time']) ? $user->format_date($row['time'], '|j F Y|, g:i a') : ''),
			'S_ACTIONS'			=> (($s_actions) ? '<div class="row-actions" data-id="' . $row_id . '">' . $s_actions . '</div>' : ''),

			'S_FOLLOW_BUTTON'	=> $follower->buttons->create('follow_button', array(
				'USER_ID'		=> $row_id,
				'USER_STATUS'	=> (!empty($row['is_following']) ? 'following' : 'follow')
			)),
			
			'S_MENTION_BUTTON'	=> $follower->buttons->create('mention', array(
				'FOLLOWNAME'	=> $row['user_follow_name'],
				'TITLE'			=> $user->lang('MENTION_TITLE', $row['user_follow_name'])
			)),
			
			'U_FOLLOWNAME'		=> $follower->u_action . 'u=' . $row['user_id'] . '&amp;mode=wall',
		));

		$final_time = (!empty($row['time']) ? $row['time'] : $final_time);
	}

	unset($wall_row);
}

function follower_bbcode_strip(&$message, $uid = '')
{
	global $phpbb_root_path;

	$accept_bbcodes = array('b', 'i', 'u', 'url', 'color');
	$find = array();

	// Replace "img" with icon to open in new window
	$message = preg_replace("#\[img:$uid\]([^\[]*)\[/img:$uid\]#", '<a class="fwicon icon-image" href="\\1" title="\\1" onclick="window.open(this.href); return false;"></a>', $message);

	// Remove all the ignore bbcodes (keep text)
	if(preg_match_all("#\[\/?([a-z0-9\*\+\-]+)(?:=(?:&quot;.*&quot;|[^\]]*))?(?::[a-z])?(\:$uid)\]#", $message, $matches))
	{
		foreach($matches[1] as $id => $match)
		{
			if(!in_array($match, $accept_bbcodes))
			{
				$find[] = $matches[0][$id];
			}
		}

		$message = str_replace($find, ' ', $message);
	}
}

function follower_notification($mode, $item_ids, $data = false, $sql_ignore_users = false)
{
	global $db, $user, $auth, $follower, $config, $phpbb_root_path, $phpEx;

	$notify_rows = $pm_users = $email_users = array();
	
	/* Compatibility with old phpbb versions */
	if($sql_ignore_users !== false && !is_array($sql_ignore_users))
	{
		$ignore_users = explode(', ', $sql_ignore_users);
		$sql_ignore_users = array();
		
		foreach($ignore_users as $user_id)
		{
			$user_id = (int)$user_id;
			$sql_ignore_users[$user_id] = $user_id;
		}
	}
	
	if(empty($sql_ignore_users))
	{
		$sql_ignore_users = array();

		// phpBB > 3.0.10
		if(version_compare(PHPBB_VERSION, '3.0.10', '>'))
		{
			if (!function_exists('phpbb_get_banned_user_ids'))
			{
				include($phpbb_root_path . 'includes/functions_user.' . $phpEx);
			}

			$sql_ignore_users = phpbb_get_banned_user_ids();
		}
		else
		{
			// Get banned User ID's (old phpBB versions)
			$sql = 'SELECT ban_userid
				FROM ' . BANLIST_TABLE . '
				WHERE ban_userid <> 0
					AND ban_exclude <> 1';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$user_id = (int) $row['ban_userid'];
				$sql_ignore_users[$user_id] = $user_id;
			}
			$db->sql_freeresult($result);
		}
		
		$sql_ignore_users[ANONYMOUS] = ANONYMOUS;
		$sql_ignore_users[$user->data['user_id']] = $user->data['user_id'];
	}

	$sql_ary = array(
		'SELECT'	=> 'u.user_id, u.user_email, u.username, u.user_lang, u.user_notify_options',
		'FROM'		=> array(),
		'LEFT_JOIN'	=> array(),
	);

	switch($mode)
	{
		case 'posts':
		case 'topics':
			$sql_ary['FROM'][FOLLOWER_TRACK_TABLE] = 'ft';

			$sql_ary['LEFT_JOIN'][] = array(
				'FROM'	=> array(USERS_TABLE => 'u'),
				'ON'	=> 'u.user_id = ft.user_id',
			);

			$sql_ary['WHERE'] = 'ft.follow_id = ' . $user->data['user_id'];

		break;

		case 'replies':
		case 'share':
			$sql_ary['FROM'][FOLLOWER_COMMENTS_TABLE] = 'c';

			$sql_ary['LEFT_JOIN'][] = array(
				'FROM'	=> array(USERS_TABLE => 'u'),
				'ON'	=> 'u.user_id = c.user_id',
			);

			$sql_ary['WHERE'] = "c.comment_id = $item_ids";

			$follower->edit($data['comment_text']);
		break;

		case 'follower':
			$sql_ary['FROM'][USERS_TABLE] = 'u';
			$sql_ary['WHERE'] = "u.user_id = $item_ids";
		break;

		case 'mentions':
			if(empty($item_ids))
			{
				return false;
			}

			$sql_ary['FROM'][USERS_TABLE] = 'u';
			$sql_ary['WHERE'] = $db->sql_in_set('u.user_id', $item_ids);
		break;

		default:
			return false;
		break;
	}

	$sql_ary['WHERE'] .= ' AND ' . $db->sql_in_set('u.user_id', $sql_ignore_users, true) . '
		AND u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')';

	$sql = $db->sql_build_query('SELECT', $sql_ary);
	
	$result = $db->sql_query($sql);
	while ($row = $db->sql_fetchrow($result))
	{
		$user_id = (int)$row['user_id'];

		$notify_rows[$user_id] = array(
			'user_id'			=> $user_id,
			'username'			=> $row['username'],
			'user_email'		=> $row['user_email'],
			'user_lang'			=> $row['user_lang'],
			'notify_options'	=> $row['user_notify_options'],
			'allowed'			=> false,
		);
	}
	$db->sql_freeresult($result);

	if(!empty($notify_rows))
	{
		// Make sure users are allowed to read the forum
		if($mode == 'posts' || $mode == 'topics')
		{
			foreach($auth->acl_get_list(array_keys($notify_rows), 'f_read', $data['forum_id']) as $forum_id => $forum_ary)
			{
				foreach ($forum_ary as $auth_option => $user_ary)
				{
					foreach ($user_ary as $user_id)
					{
						$notify_rows[$user_id]['allowed'] = true;
					}
				}
			}
		}
	
		// Now, we have to do a little step before really sending, we need to distinguish our users a little bit. ;)
		foreach($notify_rows as $user_id => $row)
		{
			if(($mode == 'posts' || $mode == 'topics') && !$row['allowed'])
			{
				unset($notify_rows[$user_id]);
				continue;
			}
	
			if($follower->notify($mode, 'pm', $row['notify_options']))
			{
				$pm_users[] = $row;
			}
			else if($follower->notify($mode, 'email', $row['notify_options']))
			{
				$email_users[] = $row;
			}

			// Dont send mention notification if previous notification was sended
			if(in_array($mode, array('posts', 'topics', 'replies')))
			{
				$sql_ignore_users[$user_id] = $user_id;
			}
		}
		
		if(!empty($notify_rows))
		{
			// Add an new notification to the followers :D
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_new_notify = user_new_notify + 1
				WHERE ' . $db->sql_in_set('user_id', array_keys($notify_rows));
			$db->sql_query($sql);
		}
	}

	$tpl_ary = array(
		'S_FOLLOWER_NAME'	=> $user->data['username'],
		'U_VIEW_TOPIC'		=> ($mode == 'topics') ? generate_board_url() . "/viewtopic.$phpEx?f={$data['forum_id']}&t={$data['topic_id']}" : '',
		'U_VIEW_POST'		=> ($mode == 'posts') ? generate_board_url() . "/viewtopic.$phpEx?f={$data['forum_id']}&p=$item_ids#p$item_ids" : '',
		'U_VIEW_PROFILE'	=> generate_board_url() . "/follower.$phpEx?u=" . $user->data['user_id'],
		'S_COMMENT_TEXT'	=> ($mode == 'replies' || $mode == 'share') ? $data['comment_text'] : '',
	);

	// Send the pm notifications if allowed
	if(sizeof($pm_users) && in_array($config['fwr_allow_notify'], array(FWR_NOTIFY_MP, FWR_NOTIFY_ALL)))
	{
		if(!function_exists('submit_pm'))
		{
			include($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);
		}

		$uid			= $bitfield			= $options		= '';
		$allow_bbcode	= $allow_smilies	= $allow_urls   = $img_status   = $flash_status = true;

		$match = $addr = array();

		foreach($pm_users as $row)
		{
			$addr[$row['user_id']] = 'bcc';
		}

		$pm_template = $phpbb_root_path . 'language/' . $user->lang_name . '/email/follower/fwr_' . $mode . '_notify.txt';
		$pm_msg = @file_get_contents($pm_template);

		if(!$pm_msg)
		{
			return false;
		}

		$pm_subject = $user->lang['NOTIFY'];
		if (preg_match('#^(Subject:(.*?))$#m', $pm_msg, $match))
		{
			if (trim($match[2]) != '')
			{
				$pm_subject = trim($match[2]);
				$pm_msg = trim(preg_replace('#[\r\n]*?' . preg_quote($match[1], '#') . '#s', '', $pm_msg));
			}
		}

		// Parse the variables
		$pm_msg = preg_replace('#\{([A-Z0-9\-_]+)\}#e', "(isset(\$tpl_ary['\$1']) ? \$tpl_ary['\$1'] : '')", $pm_msg);

		generate_text_for_storage($pm_msg, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);

		$pm_data = array(
			'address_list'			=> array('u' => $addr),
			'from_user_id'			=> $user->data['user_id'],
			'from_user_ip'			=> $user->ip,
			'from_username'			=> $user->data['username'],
			'enable_sig'			=> false,
			'enable_bbcode'			=> $allow_bbcode,
			'enable_smilies'		=> $allow_smilies,
			'enable_urls'			=> $allow_urls,
			'reply_from_root_level'	=> 0,
			'reply_from_msg_id'		=> 0,
			'icon_id'				=> 0,
			'bbcode_bitfield'		=> $bitfield,
			'bbcode_uid'			=> $uid,
			'message'				=> $pm_msg,
		);

		submit_pm('post', $pm_subject, $pm_data, false);
	}
	unset($pm_users);

	// Send the email notifications if allowed
	if(sizeof($email_users) && in_array($config['fwr_allow_notify'], array(FWR_NOTIFY_EMAIL, FWR_NOTIFY_ALL)))
	{
		if(!class_exists('messenger'))
		{
			include($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);
		}

		$messenger = new messenger();

		foreach ($email_users as $user_data)
		{
			$messenger->template('follower/fwr_' . $mode . '_notify', $user_data['user_lang']);

			$messenger->to($user_data['user_email'], $user_data['username']);

			$tpl_ary['S_USERNAME'] = htmlspecialchars_decode($user_data['username']);

			$messenger->assign_vars($tpl_ary);
			$messenger->send();
		}

		unset($email_users);

		$messenger->save_queue();
	}
	
	// Send mention notification
	if(in_array($mode, array('posts', 'topics', 'replies')))
	{
		follower_notification('mentions', $follower->mention_ids, false, $sql_ignore_users);
	}

	unset($notify_rows);
}

function follower_get_following_users($member)
{
	global $db;
	static $fwr_following_users;
	
	if(empty($fwr_following_users))
	{
		$fwr_following_users = array();

		// Get following_users
		$sql = 'SELECT follow_id
			FROM ' . FOLLOWER_TRACK_TABLE . '
			WHERE user_id = ' . $member['user_id'] . '
			AND follower = 1';
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$fwr_following_users[] = $row['follow_id'];
		}
		$db->sql_freeresult($result);
	}
	
	return $fwr_following_users;
}

function follower_get_who_follow($member)
{
	global $db, $template, $user, $follower;
	
	$following_users = $user_list = $user_cache = array();
	$ignore_users = array($user->data['user_id']);
	
	// Get following by member
	$following_users = follower_get_following_users($member);
	
	if(sizeof($following_users))
	{
		$ignore_users = array_merge($ignore_users, $following_users);

		// Get aleatory following from the member following
		$sql = 'SELECT user_id, follow_id
			FROM ' . FOLLOWER_TRACK_TABLE . '
			WHERE ' . $db->sql_in_set('user_id', $following_users) . '
			AND ' . $db->sql_in_set('follow_id', $ignore_users, true) . '
			AND follower = 1
			ORDER BY RAND()';
		$result = $db->sql_query_limit($sql, 10);
		while($row = $db->sql_fetchrow($result))
		{
			$user_list[] = $row;
			$ignore_users[] = $row['follow_id'];
			
			$user_cache[$row['user_id']] 	= $row['user_id'];
			$user_cache[$row['follow_id']] 	= $row['follow_id'];
		}
		$db->sql_freeresult($result);
	}
	
	// Get users with more posts
	$sql = 'SELECT user_id AS poster_id
		FROM ' . USERS_TABLE . '
		WHERE ' . $db->sql_in_set('user_id', $ignore_users, true) . '
		AND user_type <> ' . USER_IGNORE . '
		AND user_posts > 0
		ORDER BY user_posts DESC';
	$result = $db->sql_query_limit($sql, 10);
	while($row = $db->sql_fetchrow($result))
	{
		$user_list[] = $row;
		$user_cache[$row['poster_id']] 	= $row['poster_id'];
	}
	$db->sql_freeresult($result);
	
	if(!sizeof($user_list))
	{
		return false;
	}
	
	// Get user information
	$sql = 'SELECT user_id, username, user_follow_name,
		user_avatar, user_avatar_type, user_avatar_width, user_avatar_height
		FROM ' . USERS_TABLE . '
		WHERE ' . $db->sql_in_set('user_id', $user_cache);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$user_cache[$row['user_id']] = $row;
	}
	$db->sql_freeresult($result);
	
	$rand_keys = array_rand($user_list, min(10, count($user_list)));
	
	if(!is_array($rand_keys))
	{
		$rand_keys = array($rand_keys);
	}
	
	// Output the users
	foreach($rand_keys as $key)
	{
		$row = $user_list[$key];

		if(isset($row['poster_id']))
		{
			$follow_row	= $user_cache[$row['poster_id']];
		}
		else
		{
			$user_row 	= $user_cache[$row['user_id']];
			$follow_row	= $user_cache[$row['follow_id']];
		}
		
		$template->assign_block_vars('who_follow', array(
			'USER_ID'			=> $follow_row['user_id'],
			'S_AVATAR'			=> ($user->optionget('viewavatars')) ? get_user_avatar($follow_row['user_avatar'], $follow_row['user_avatar_type'], $follow_row['user_avatar_width'], $follow_row['user_avatar_height']) : '',
			'S_FOLLOWNAME'		=> get_followname_string('label', $follow_row['user_follow_name'], $follow_row['user_id']),
			'S_FULLNAME'		=> $follow_row['username'],
			'S_INFO'			=> (isset($row['poster_id']) ? $user->lang['USUAL_POSTER'] : $user->lang('FOLLOWING_BY', $user_row['username'])),
			
			'S_FOLLOW_BUTTON'	=> $follower->buttons->create('follow_button', array(
				'USER_ID'		=> $follow_row['user_id'],
				'USER_STATUS'	=> 'fwsmall fwnoicon follow'
			)),
			
			'U_FOLLOWNAME'		=> $follower->u_action . 'u=' . $follow_row['user_id'] . '&amp;mode=wall',
		));
	}
	
	unset($user_cache, $user_list);
}

function follower_get_trends()
{
	global $db, $template;
	
	$trends_ary = array();

	$sql = 'SELECT COUNT(hashtag) AS count, hashtag
		FROM ' . FOLLOWER_HASHTAGS_TABLE . '
		GROUP BY hashtag
		ORDER BY count DESC';
	$result = $db->sql_query_limit($sql, 10, 0, 3600); // Cached during 1 hour
	while($row = $db->sql_fetchrow($result))
	{
		if($row['count'] < 2)
		{
			break;
		}

		$trends_ary[] = $row['hashtag'];
	}
	$db->sql_freeresult($result);
	
	foreach($trends_ary as $hashtag)
	{
		$template->assign_block_vars('trendrow', array(
			'HASHTAG'	=> get_hashtag_string($hashtag),
		));
	}
	
	unset($trends_ary);
}

/**
 * html_truncate
 * Truncates string while retaining special characters if going over the max length
 *
 * @param string	$input . The text to truncate to the given length.
 * @param int		$limit . Maximum length of string
 * @param string	$append . String to be appended
 * @param bool		$count_all . Defines if count all or only plain text
*/
if(!function_exists('html_truncate'))
{
	function html_truncate($input = '', $limit = 0, $append = '', $count_all = false)
	{
		$check_len = ($count_all) ? utf8_strlen($input) : utf8_strlen(strip_tags($input));
		if(!$input || $limit <= 0 || $check_len <= $limit)
		{
			return $input;
		}

		$splits = preg_split('#(</?[^>]*/?>)#', $input, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

		$text = '';
		$open_tag = false;
		foreach($splits as $split)
		{
			// If no open tag, open it if necessary
			if($open_tag === false)
			{
				if(preg_match('#^<([a-zA-Z]+).*>#', $split, $match))
				{
					if(utf8_substr($match[0], -2) !== '/>')
					{
						$open_tag = $match[1];
					}
				}
			}

			// Close tag, if the current
			if($open_tag !== false)
			{
				if($split === "</$open_tag>")
				{
					$open_tag = false;
				}
			}

			// Adjust the string
			if($open_tag === false && $split[0] != '<')
			{ 
				$length = ($count_all) ? utf8_strlen($text) : utf8_strlen(strip_tags($text));

				if(($length + utf8_strlen($split)) > $limit)
				{
					$split = utf8_substr($split, 0, $limit - $length);
				}
			}

			$text .= $split;

			// If exceed the limit and no open tags finish here,
			// and it is looking backwards to remove the tag open.
			$check_len = ($count_all) ? utf8_strlen($text) : utf8_strlen(strip_tags($text));
			if($check_len >= $limit)
			{
				if($open_tag !== false)
				{
					$tag = '<' . $open_tag; 
					$text = utf8_substr($text, 0, utf8_strrpos($text, $tag));
					break;
				}
				break;
			}
		}

		return $text . (($append) ? $append : '');
	}
}

?>
