<?php
/*
*
* @name functions_follower_core.php
* @package phpBB3 cBB Follower
* @version $Id: functions_follower_core.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

function do_core_action()
{
	global $db, $config, $user, $auth, $template, $phpbb_root_path, $phpEx;
	global $follower, $wall_row, $user_cache, $comment_track_cache;

	$action = request_var('action', '');
	$id		= request_var('id', 0);
	$start	= request_var('start', 0);
	$u		= request_var('u', 0);
	$mode	= request_var('mode', '');

	$row_limit	= $config['fwr_row_limit'];
	$myself		= ($u == $user->data['user_id']) ? true : false;

	$json = array();
	
	// Do nothing if follower isn't active
	if(!$follower->active)
	{
		$action = '';
	}
	
	if(!function_exists('follower_notification'))
	{
		include($phpbb_root_path . 'follower/includes/functions_follower.' . $phpEx);
	}
	
	if(!function_exists('get_user_avatar'))
	{
		include($phpbb_root_path . 'includes/functions_display.' . $phpEx);
	}

	switch($action)
	{
		case 'follow':
		case 'unfollow':
			if(!$follower->can_follow)
			{
				break;
			}

			$sql = array();

			$sql_ary = array(
				'user_id'		=> $user->data['user_id'],
				'follow_id'		=> $id,
				'follower'		=> 1,
			);

			$query_set = '';

			if($action == 'follow')
			{
				$sql_ary['track_time'] = time();

				$sql[] = 'INSERT INTO ' . FOLLOWER_TRACK_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
				$query_set = '+ 1';
			}
			else
			{
				$sql[] = 'DELETE FROM ' . FOLLOWER_TRACK_TABLE . '
					WHERE ' . $db->sql_build_array('SELECT', $sql_ary);
				$query_set = '- 1';
			}

			// Update the users values
			$sql[] = 'UPDATE ' . USERS_TABLE . "
				SET user_following = user_following $query_set
				WHERE user_id = " . $user->data['user_id'] .
				(($action == 'unfollow') ? ' AND user_following > 0' : '');

			$sql[] = 'UPDATE ' . USERS_TABLE . "
				SET user_followers = user_followers $query_set
				WHERE user_id = " . $id . 
				(($action == 'unfollow') ? ' AND user_followers > 0' : '');

			foreach($sql as $_sql)
			{
				$db->sql_query($_sql);
			}

			// Send notification to user
			if($action == 'follow')
			{
				follower_notification('follower', $id);
			}

			if($myself)
			{
				$json['following-count'] = ($action == 'follow') ? $user->data['user_following'] + 1 : $user->data['user_following'] - 1;
				
				if($action == 'unfollow')
				{
					$json['delete-row'] = true;
				}
			}
		break;

		case 'reply-comment':
			if(!$auth->acl_get('u_fwr_share'))
			{
				break;
			}

			$user_ids = array();
			
			// Get author of the comment
			$sql = 'SELECT user_id
				FROM ' . FOLLOWER_COMMENTS_TABLE . "
				WHERE comment_id = $id";
			$result = $db->sql_query($sql);
			$author_id = (int)$db->sql_fetchfield('user_id');
			$db->sql_freeresult($result);
			
			$user_ids[$author_id] = $author_id;
			
			// Get mention names in the comment
			$sql = 'SELECT user_id
				FROM ' . FOLLOWER_MENTIONS_TABLE . "
				WHERE comment_id = $id";
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$user_ids[$row['user_id']] = (int)$row['user_id'];
			}
			$db->sql_freeresult($result);
			
			// Exclude this user
			unset($user_ids[$user->data['user_id']]);
			
			// Get follower names
			if(sizeof($user_ids))
			{
				$sql = 'SELECT user_id, user_follow_name
					FROM ' . USERS_TABLE . '
					WHERE ' . $db->sql_in_set('user_id', $user_ids);
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$user_ids[$row['user_id']] = '@' . $row['user_follow_name'];
				}
				$db->sql_freeresult($result);
			}

			$json = array(
				'title'		=> $user->lang['COMMENT_REPLY'],
				'msg'		=> (sizeof($user_ids) ? implode(' ', $user_ids) . ' ' : ''),

				'btn-data'	=> 'mode=' . $mode . '&id=' . $id,
			);
		break;

		case 'delete-comment':
			$sql = 'SELECT user_id
				FROM ' . FOLLOWER_COMMENTS_TABLE . "
				WHERE comment_id = $id";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
			
			if(!$row)
			{
				break;	
			}
			
			$is_mine = ($row['user_id'] == $user->data['user_id']) ? true : false;
	
			if((!$is_mine && !$auth->acl_get('m_fwr_delete')) || ($is_mine && !$auth->acl_get('u_fwr_delete')))
			{
				break;
			}
			
			$delete_tables = array(
				FOLLOWER_COMMENTS_TABLE		=> '',
				FOLLOWER_HASHTAGS_TABLE		=> '',
				FOLLOWER_MENTIONS_TABLE		=> '',
				FOLLOWER_TRACK_TABLE		=> 'share = 1',
			);
			
			foreach($delete_tables as $table => $extra_sql)
			{
				$sql = "DELETE FROM $table
					WHERE comment_id = $id" .
					(($extra_sql) ? " AND $extra_sql" : '');
				$db->sql_query($sql);
			}

			$db->sql_query('UPDATE ' . FOLLOWER_COMMENTS_TABLE . " SET parent_id = 0 WHERE parent_id = $id");

			$db->sql_query('UPDATE ' . USERS_TABLE . '
				SET user_comments = user_comments - 1
				WHERE user_id = ' . $row['user_id']);

			if($myself)
			{
				$json['comment-count'] = $user->data['user_comments'] - 1;
			}

			$json['l-noitems'] = ($myself) ? $user->lang['EMPTY_MY_' . strtoupper($mode)] : $user->lang('EMPTY_OTHER_' . strtoupper($mode), $u);
		break;

		case 'submit-comment':
			if(!$auth->acl_get('u_fwr_publish'))
			{
				break;
			}

			$message = utf8_normalize_nfc(request_var('message', '', true));

			if($message)
			{
				$follower->parse($message);

				$sql_ary = array(
					'user_id'		=> $user->data['user_id'],
					'parent_id'		=> $id,
					'comment_text'	=> $message,
					'comment_time'	=> time(),
				);

				$sql = 'INSERT INTO ' . FOLLOWER_COMMENTS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);

				if(!$db->sql_query($sql))
				{
					break;
				}

				$comment_id = (int)$db->sql_nextid();

				// Update de value to the user
				$db->sql_query('UPDATE ' . USERS_TABLE . '
					SET user_comments = user_comments + 1
					WHERE user_id = ' . $user->data['user_id']);

				$follower->store(0, $comment_id, $sql_ary['comment_time']);

				// Send notification
				if($id)
				{
					follower_notification('replies', $id, array('comment_text' => $message));
				}
				else
				{
					follower_notification('mentions', $follower->mention_ids);
				}

				if($mode == 'wall' && $myself)
				{
					$wall_row = array();
	
					$wall_row[] = array(
						'id'		=> 'comment-' . $comment_id,
						'user_id'	=> $sql_ary['user_id'],
						'parent_id'	=> $sql_ary['parent_id'],
						'text'		=> $sql_ary['comment_text'],
						'time'		=> $sql_ary['comment_time'],
					);


					$user_cache[$sql_ary['user_id']] = $sql_ary['user_id'];
					$comment_track_cache[$comment_id] = false;
					
					display_follower_rows($wall_row);
				}
			}

			// Prune old comments if applicable (10 comments per time)
			if($config['fwr_store_time'])
			{
				$allowed_time = time() - (int)$config['fwr_store_time'];
				$substracts = $comment_ids = array();
				
				// Select the oldest comments to delete aund its users
				$sql = 'SELECT user_id, comment_id
					FROM ' . FOLLOWER_COMMENTS_TABLE . "
					WHERE comment_time < $allowed_time
					ORDER BY comment_time";
				$result = $db->sql_query($sql, 10);
				while($row = $db->sql_fetchrow($result))
				{
					if(!isset($substracts[$row['user_id']]))
					{
						$substracts[$row['user_id']] = 1;
					}
					else
					{
						$substracts[$row['user_id']]++;
					}
			
					$comment_ids[] = $row['comment_id'];
				}
				$db->sql_fetchrow($result);
				
				if(sizeof($comment_ids))
				{
					// Delete the comments
					$sql = 'DELETE FROM ' . FOLLOWER_COMMENTS_TABLE . '
						WHERE ' . $db->sql_in_set('comment_id', $comment_ids);
					$db->sql_query($sql);
					
					// Set the replies as new comment
					$sql = 'UPDATE ' . FOLLOWER_COMMENTS_TABLE . '
						SET parent_id = 0
						WHERE ' . $db->sql_in_set('parent_id', $comment_ids);
					$db->sql_query($sql);
					
					// Update the comment count of the users
					foreach($substracts as $user_id => $substract)
					{
						$sql = 'UPDATE ' . USERS_TABLE . "
							SET user_comments = user_comments - $substract
							WHERE user_id = $user_id";
						$db->sql_query($sql);
					}
				}
			}
		break;

		case 'share-comment':
		case 'unshare-comment':
			if(!$auth->acl_get('u_fwr_share'))
			{
				break;
			}

			$sql_ary = array(
				'user_id'		=> $user->data['user_id'],
				'comment_id'	=> $id,
				'share'			=> 1,
			);

			if($action == 'share-comment')
			{
				$sql_ary['track_time'] = time();

				$sql = 'INSERT INTO ' . FOLLOWER_TRACK_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);

				$json['msg'] = $user->lang[$follower->row_actions['unshare']['lang']];
			}
			else
			{
				$sql = 'DELETE FROM ' . FOLLOWER_TRACK_TABLE . '
					WHERE ' . $db->sql_build_array('SELECT', $sql_ary);

				$json['msg'] = $user->lang[$follower->row_actions['share']['lang']];
			}

			$db->sql_query($sql);

			// Send user notification
			if ($action == 'share-comment')
			{
				$comment_id = (int)$db->sql_nextid();
				follower_notification('share', $comment_id);
			}
		break;
		
		case 'finduser':
			if(!$auth->acl_get('u_fwr_mention'))
			{
				break;
			}
			
			$query = utf8_normalize_nfc(request_var('q', '', true));
			
			$sql = 'SELECT *
				FROM ' . USERS_TABLE . '
				WHERE (
					LOWER(user_follow_name) ' . $db->sql_like_expression(strtolower($query) . $db->any_char) . '
					OR username_clean ' . $db->sql_like_expression(utf8_clean_string($query) . $db->any_char) . '
				)
				AND user_type <> ' . USER_IGNORE . '
				ORDER BY user_follow_name ASC';
			$result = $db->sql_query_limit($sql, 50);
			while($row = $db->sql_fetchrow($result))
			{
				$template->assign_block_vars('userrow', array(
					'S_AVATAR'			=> ($user->optionget('viewavatars')) ? get_user_avatar($row['user_avatar'], $row['user_avatar_type'], $row['user_avatar_width'], $row['user_avatar_height']) : '',
					'S_FOLLOWNAME'		=> '@' . $row['user_follow_name'],
					'S_USERNAME'		=> $row['username'],
				));
			}
			$db->sql_freeresult($result);
			
			$template->assign_var('IN_USERLIST', true);
		break;

		case 'more-info':
			$sql = 'SELECT parent_id
				FROM ' . FOLLOWER_COMMENTS_TABLE . "
				WHERE comment_id = $id";
			$result = $db->sql_query($sql);
			$parent_id = (int)$db->sql_fetchfield('parent_id');
			$db->sql_freeresult($result);

			$sql = 'SELECT *
				FROM ' . FOLLOWER_COMMENTS_TABLE . "
				WHERE parent_id = $id
				OR " . $db->sql_in_set('comment_id', array($id, $parent_id)) . '
				ORDER BY comment_time ASC';
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$wall_row[] = array(
					'id'			=> 'comment-' . $row['comment_id'],
					'user_id'		=> $row['user_id'],
					'parent_id'		=> $row['parent_id'],

					'text'			=> $row['comment_text'],
					'time'			=> $row['comment_time'],
				);

				$user_cache[$row['user_id']] = $row['user_id'];
				$comment_track_cache[$row['comment_id']] = false;
			}
			$db->sql_freeresult($result);

			display_follower_rows($wall_row, false, '', $id);
		break;
		
		case 'view_notify':
			$notify_row = $following_users = $user_cache = array();
			
			$start_time	= request_var('time', time());
			$final_time = $start_time;

			$next_page	= (isset($_POST['next']) ? true : false);
			$row_limit	= ($next_page) ? 10 : $config['fwr_row_limit'];
			
			$forum_ids = array_keys($auth->acl_getf('f_read', true));
	
			// Get following and follower users
			$sql = 'SELECT user_id, follow_id, track_time
				FROM ' . FOLLOWER_TRACK_TABLE . '
				WHERE (
					(user_id = ' . $user->data['user_id'] . "
					AND track_time < $start_time)
					OR follow_id = " . $user->data['user_id'] . ')
				AND follower = 1';
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				if($row['user_id'] == $user->data['user_id'])
				{
					// if is following, cache to the next queries
					$following_users[] = $row['follow_id'];
				}
				else
				{
					// if is follower, add to the display rows
					unset($row['follow_id']);

					$notify_row[$row['track_time']] = $row;
					$user_cache[$row['user_id']] = $row['user_id'];
				}
			}
			$db->sql_freeresult($result);

			if(!empty($following_users))
			{
				// Get the posts
				$sql = 'SELECT p.post_id, p.poster_id, p.post_subject, p.post_time
					FROM ' . POSTS_TABLE . ' p
						LEFT JOIN ' . TOPICS_TABLE . ' t ON(t.topic_id = p.topic_id)
					WHERE ' . $db->sql_in_set('p.poster_id', $following_users) . "
					AND p.post_approved = 1
					AND p.post_time < $start_time
					AND " . $db->sql_in_set('p.forum_id', $forum_ids) . '
						AND t.topic_first_post_id <> p.post_id
					ORDER BY p.post_time DESC';
				$result = $db->sql_query_limit($sql, $row_limit);
				while($row = $db->sql_fetchrow($result))
				{
					$notify_row[$row['post_time']] = $row;
					$user_cache[$row['poster_id']] = $row['poster_id'];
				}
				$db->sql_freeresult($result);
		
				// Get the topics
				$sql = 'SELECT t.topic_id, t.topic_title, t.topic_time, t.topic_poster
					FROM ' . TOPICS_TABLE . ' t
					WHERE ' . $db->sql_in_set('t.topic_poster', $following_users) . '
					AND t.topic_approved = 1
					AND t.topic_status <> ' . ITEM_MOVED . '
					AND ' . $db->sql_in_set('t.forum_id', $forum_ids) . "
					AND t.topic_time < $start_time
					ORDER BY t.topic_time DESC";
				$result = $db->sql_query_limit($sql, $row_limit);
				while($row = $db->sql_fetchrow($result))
				{
					$notify_row[$row['topic_time']] = $row;
					$user_cache[$row['topic_poster']] = $row['topic_poster'];
				}
				$db->sql_freeresult($result);
		
				if ($config['fwr_allow_comments'])
				{
					// Get the comments (shared by following users)
					$sql = 'SELECT fc.comment_id, fc.user_id, fc.comment_time, ft.user_id AS share_id
						FROM ' . FOLLOWER_TRACK_TABLE . ' ft
						LEFT JOIN ' . FOLLOWER_COMMENTS_TABLE . ' fc ON(fc.comment_id = ft.comment_id)
						WHERE ' . $db->sql_in_set('ft.user_id', $following_users) . "
						AND ft.share = 1
						AND ft.track_time < $start_time
						AND ft.user_id <> " . $user->data['user_id'] . '
						ORDER BY ft.track_time DESC';
					$result = $db->sql_query_limit($sql, $row_limit);
		
					while($row = $db->sql_fetchrow($result))
					{
						$notify_row[$row['comment_time']] = $row;
						$user_cache[$row['user_id']] = $row['user_id'];
						$user_cache[$row['share_id']] = $row['share_id'];
					}
					$db->sql_freeresult($result);
		
					// Get the comment from the following users
					$sql = 'SELECT comment_id, user_id, comment_time
						FROM ' . FOLLOWER_COMMENTS_TABLE . '
						WHERE ' . $db->sql_in_set('user_id', $following_users) . "
						AND comment_time < $start_time
						ORDER BY comment_time DESC";
					$result = $db->sql_query_limit($sql, $row_limit);
		
					while($row = $db->sql_fetchrow($result))
					{
						$notify_row[$row['comment_time']] = $row;
						$user_cache[$row['user_id']] = $row['user_id'];
					}
					$db->sql_freeresult($result);
				}

				// Get following of my following
				$sql = 'SELECT user_id, follow_id, track_time
					FROM ' . FOLLOWER_TRACK_TABLE . '
					WHERE ' . $db->sql_in_set('user_id', $following_users) . '
					AND follower = 1
					AND follow_id <> ' . $user->data['user_id'] . "
					AND track_time < $start_time";
				$result = $db->sql_query_limit($sql, $row_limit);
				while($row = $db->sql_fetchrow($result))
				{
					$notify_row[$row['track_time']] = $row;
	
					$user_cache[$row['user_id']] = $row['user_id'];
					$user_cache[$row['follow_id']] = $row['follow_id'];
				}
				$db->sql_freeresult($result);
			}

			// Get the mentions
			$sql = 'SELECT fm.comment_id, fm.post_id, fm.mention_time,
				fc.user_id, p.poster_id, p.post_subject
				FROM ' . FOLLOWER_MENTIONS_TABLE . ' fm
					LEFT JOIN ' . FOLLOWER_COMMENTS_TABLE . ' fc
						ON(fc.comment_id = fm.comment_id AND fc.user_id <> ' . $user->data['user_id'] . ')
					LEFT JOIN ' . POSTS_TABLE . ' p
						ON(p.post_id = fm.post_id AND p.poster_id <> ' . $user->data['user_id'] . ')
				WHERE fm.user_id = ' . $user->data['user_id'] . "
				AND fm.mention_time < $start_time";
			$result = $db->sql_query_limit($sql, $row_limit);
			while($row = $db->sql_fetchrow($result))
			{
				if($row['poster_id'] || $row['user_id'])
				{
					$notify_row[$row['mention_time']] = $row;
	
					if($row['post_id'])
					{
						$user_cache[$row['poster_id']] = $row['poster_id'];
					}
					else
					{
						$user_cache[$row['user_id']] = $row['user_id'];
					}
				}
			}
			$db->sql_freeresult($result);
	
			if(!empty($notify_row))
			{
				// Get the user information
				$sql = 'SELECT user_id, user_follow_name, username
					FROM ' . USERS_TABLE . '
					WHERE ' . $db->sql_in_set('user_id', array_keys($user_cache));
				$result = $db->sql_query($sql);
			
				while($row = $db->sql_fetchrow($result))
				{
					$user_cache[$row['user_id']] = $row;
				}
				$db->sql_freeresult($result);

				// Proccess the rows
				$rows = 0;
				$row_limit = min($row_limit, count($notify_row));

				// Sort the rows
				krsort($notify_row);
	
				foreach($notify_row as $row_time => $row)
				{
					if($rows >= $row_limit)
					{
						break;	
					}
					
					$rows++;

					if(isset($row['mention_time']))
					{
						// Mentions to me
						if($row['post_id'])
						{
							$u_action = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'p=' . $row['post_id'] . '#' . $row['post_id']);
							$content = $user->lang('NOTIFY_MENTION_POST', get_followname_string('full', $user_cache[$row['poster_id']]), $u_action, $row['post_subject']);
						}
						else
						{
							$content = $user->lang('NOTIFY_MENTION_COMMENT', get_followname_string('full', $user_cache[$row['user_id']]));
						}
					}
					else if(isset($row['post_id']))
					{
						// New post from a following user
						$u_action = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'p=' . $row['post_id'] . '#' . $row['post_id']);
						$content = $user->lang('NOTIFY_NEW_POST', get_followname_string('full', $user_cache[$row['poster_id']]), $u_action, $row['post_subject']);
					}
					else if(isset($row['topic_id']))
					{
						// New topic from a following user
						$u_action = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 't=' . $row['topic_id']);
						$content = $user->lang('NOTIFY_NEW_TOPIC', get_followname_string('full', $user_cache[$row['topic_poster']]), $u_action, $row['topic_title']);
					}
					else if(isset($row['comment_id']))
					{
						// New comment or share from a following user
						if(isset($row['share_id']))
						{
							$content = $user->lang('NOTIFY_NEW_SHARE', get_followname_string('full', $user_cache[$row['share_id']]), get_followname_string('full', $user_cache[$row['user_id']]));
						}
						else
						{
							$content = $user->lang('NOTIFY_NEW_COMMENT', get_followname_string('full', $user_cache[$row['user_id']]));
						}
					}
					else if(!isset($row['follow_id']))
					{
						// New follower	
						$content = $user->lang('NOTIFY_NEW_FOLLOWER', get_followname_string('full', $user_cache[$row['user_id']]));
					}
					else
					{
						// My following users following another user
						$content = $user->lang('NOTIFY_NEW_FOLLOWING_OF_FOLLOWING', get_followname_string('full', $user_cache[$row['user_id']]), get_followname_string('full', $user_cache[$row['follow_id']]));
					}
	
					$template->assign_block_vars('notifyrow', array(
						'S_CONTENT'		=> $content,
						'S_TIME'		=> $user->format_date($row_time),
					));
					
					$final_time = $row_time;
				}
			}
			
			// Mark the notifications as read
			if($user->data['user_new_notify'])
			{
				$sql = 'UPDATE ' . USERS_TABLE . '
					SET user_new_notify = 0
					WHERE user_id = ' . $user->data['user_id'];
				$db->sql_query($sql);
			}
			
			$template->assign_vars(array(
				'IN_NOTIFY'		=> true,
				'S_IN_NEXT'		=> $next_page,
				'U_NEXT_ACTION'	=> ($row_limit == count($notify_row)) ? '' : 'next=1&action=' . $action . '&time=' . $final_time,
			));
			
			unset($notify_row);
		break;

		default:
			$action = '';
		break;
	}

	if(!empty($json))
	{
		$template->assign_var('S_JSON_RESULT', json_encode($json));
	}

	if($action)
	{
		// Determine board url - we may need it later
		$board_url = generate_board_url() . '/';
		$web_path = (defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) ? $board_url : $phpbb_root_path;

		$template->assign_vars(array(
			'S_READMODE'	=> true,
			'T_THEME_PATH'	=> "{$web_path}styles/" . $user->theme['theme_path'] . '/theme',
		));

		$template->set_filenames(array(
			'body' => 'follower/follower_body.html'
		));
		$template->display('body');
	}

	garbage_collection();
	exit_handler();
}

if(!function_exists('json_encode'))
{
	function json_encode($arr)
	{
		$parts		= array();
		$is_list	= false;
		$keys		= array_keys($arr);
		$max_length	= count($arr) - 1;

		// See if the first key is 0 and last key is length - 1
		if($keys[0] == 0 && $keys[$max_length] == $max_length)
		{
			$is_list = true;

			for ($i = 0; $i < count($keys); $i++)
			{
				if($i != $keys[$i])
				{
					$is_list = false; // It is an associative array.
					break;
				}
			}
		}

		foreach($arr as $key => $value)
		{
			$str = (($is_list) ? '' : "'$key':");

			// Custom handling for arrays
			if(is_array($value))
			{ 
				$parts[] = $str . json_encode($value); /* :RECURSION: */
			}
			else
			{
				// Custom handling for multiple data types
				if(is_numeric($value))
				{
					$str .= $value; // Numbers
				}
				else if($value === false)
				{
					$str .= 'false'; // Booleans
				}
				else if($value === true)
				{
					$str .= 'true';
				}
				else
				{
					$str .= '"' . addslashes($value) . '"'; // All other things
				}

				$parts[] = $str;
			}
		}

		$json = implode(',', $parts);

		// Return numerical JSON
		if($is_list)
		{
			return '[' . $json . ']';
		}

		// Return associative JSON
		return '{' . $json . '}';
	}
}

?>
