<?php
/*
*
* @name search.php
* @package phpBB3 cBB Follower
* @version $Id: search.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class follower_search
{
	var $page_title = '';

	function do_search()
	{
		$search_type	= request_var('type', '');
		$keywords		= trim(utf8_normalize_nfc(request_var('keywords', '', true)));

		if($keywords[0] == '@' || $keywords[0] == '#')
		{
			$search_type	= ($keywords[0] == '@') ? 'user' : 'hashtag';
			$keywords		= substr($keywords, 1, utf8_strlen($keywords));
		}
	
		switch($search_type)
		{
			case 'hashtag':
				$this->_search_hashtag($keywords, true);
			break;
			
			case 'user':
				$this->_search_users($keywords, true);
				$this->_search_hashtag($keywords);
			break;

			default:
				$this->_search_keywords($keywords);
				$this->_search_users($keywords);
				$this->_search_hashtag($keywords);
			break;
		}
		
		$this->page_title = 'SEARCH_RESULTS';
	}
	
	function display()
	{
		global $user, $template;
		global $wall_row, $next_page, $bbcode_bitfield;
		
		if(sizeof($wall_row))
		{
			display_follower_rows($wall_row, $next_page, $bbcode_bitfield);
		}

		$template->set_filenames(array(
			'body' => 'follower/follower_body.html'
		));
		
		$page_title = (isset($user->lang[$this->page_title]) ? $user->lang[$this->page_title] : $this->page_title);

		page_header($page_title);
		
		$template->assign_vars(array(
			'S_IN_SEARCH'		=> true,
			'S_READMODE'		=> ($next_page) ? true : false,
		));

		page_footer();
	}
	
	function _search_hashtag($hashtag, $main_search = false)
	{
		global $db, $template, $user;
		global $start_time, $forum_ids, $row_limit, $next_page, $bbcode_bitfield;
		global $wall_row, $user_cache, $comment_track_cache;
	
		$hashtag_row = $post_rows = $comment_rows = array();
		
		if($hashtag)
		{
			if($main_search)
			{
				$sql = 'SELECT comment_id, post_id
					FROM ' . FOLLOWER_HASHTAGS_TABLE . "
					WHERE hashtag = '" . $db->sql_escape($hashtag) . "'";
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					if($row['comment_id'])
					{
						$comment_rows[$row['comment_id']] = $row['comment_id'];
					}
					else
					{
						$post_rows[$row['post_id']] = $row['post_id'];
					}
				}
				$db->sql_freeresult($result);
				
				// Get Comments
				if(sizeof($comment_rows))
				{
					$sql = 'SELECT comment_id, user_id, parent_id, comment_text AS text, comment_time AS time
						FROM ' . FOLLOWER_COMMENTS_TABLE . '
						WHERE ' . $db->sql_in_set('comment_id', $comment_rows) . "
						AND comment_time < $start_time
						ORDER BY comment_time DESC";
					$result = $db->sql_query_limit($sql, $row_limit);

					while($row = $db->sql_fetchrow($result))
					{
						$hashtag_row[$row['time']] = $row;
					}
					$db->sql_freeresult($result);
					
					unset($comment_rows);
				}
				
				// Get Posts
				if(sizeof($post_rows))
				{
					$sql = 'SELECT p.post_id, p.poster_id AS user_id, p.post_subject, p.post_text AS text, p.post_time AS time, p.bbcode_uid, p.bbcode_bitfield
						FROM ' . POSTS_TABLE . ' p
						WHERE ' . $db->sql_in_set('p.post_id', $post_rows) . "
						AND p.post_approved = 1
						AND p.post_time < $start_time
						AND " . $db->sql_in_set('p.forum_id', $forum_ids) . '
						ORDER BY p.post_time DESC';
					$result = $db->sql_query_limit($sql, $row_limit);
					while($row = $db->sql_fetchrow($result))
					{
						$hashtag_row[$row['time']] = $row;
					}
					$db->sql_freeresult($result);
					
					unset($post_rows);
				}
				
				if(empty($hashtag_row))
				{
					return false;
				}

				// Sort the rows
				krsort($hashtag_row);

				$limit = min($row_limit, count($hashtag_row));

				// Proccess
				while(sizeof($wall_row) < $limit && !empty($hashtag_row))
				{
					$row = array_shift($hashtag_row);

					$row_id = 0;
					$row_type = $row_text = '';

					if(isset($row['post_id']))
					{
						$row_id			= $row['post_id'];
						$row_type		= 'post';
					}
					else
					{
						$row_id		= $row['comment_id'];
						$row_type	= 'comment';
						$comment_track_cache[$row['comment_id']] = false;
					}

					$row_time	= (isset($row['time']) ? $row['time'] : 0);
					$is_shared	= !empty($row['share_id']);

					$wall_row[$row_time] = array(
						'id'				=> (isset($row['id']) ? $row['id'] : $row_type . '-' . $row_id),
						'user_id'			=> $row['user_id'],

						'is_share'			=> $is_shared,
						'share_id'			=> ($is_shared) ? $row['share_id'] : 0,
						'parent_id'			=> !empty($row['parent_id']),

						'post_id'			=> (isset($row['post_id']) ? $row['post_id'] : 0),
						'topic_id'			=> (isset($row['topic_id']) ? $row['topic_id'] : 0),
						'forum_id'			=> (isset($row['forum_id']) ? $row['forum_id'] : 0),
						'forum_name'		=> (isset($row['forum_name']) ? $row['forum_name'] : 0),
						'topic_replies'		=> (isset($row['topic_replies']) ? $row['topic_replies'] : 0),
						'post_subject'		=> (isset($row['post_subject']) ? $row['post_subject'] : ''),

						'text'				=> $row['text'],
						'time'				=> $row_time,
					);

					if(isset($row['bbcode_bitfield']))
					{
						$wall_row[$row_time] += array(
							'bbcode_uid'		=> $row['bbcode_uid'],
							'bbcode_bitfield'	=> $row['bbcode_bitfield'],
						);

						$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
					}

					$user_cache[$row['user_id']] = $row['user_id'];

					if($is_shared)
					{
						$user_cache[$row['share_id']] = $row['share_id'];
					}
				}

				unset($hashtag_row);
			}

			follower_get_trends();
		}
		
		if($main_search)
		{
			$template->assign_var('S_SEARCH_TITLE', $user->lang('RESULTS_OF', '#' . $hashtag));
		}
	}
	
	function _search_keywords($keywords)
	{
		global $db, $template, $user, $start_time, $row_limit;
		global $wall_row, $user_cache, $comment_track_cache, $next_page;
		
		$keywords = trim($keywords);
		
		if($keywords)
		{
			$sql = 'SELECT comment_id, user_id, parent_id, comment_text AS text, comment_time AS time
				FROM ' . FOLLOWER_COMMENTS_TABLE . '
				WHERE LOWER(comment_text) ' . $db->sql_like_expression($db->any_char . strtolower($keywords) . $db->any_char) . "
				AND comment_time < $start_time
				ORDER BY comment_time DESC";
			$result = $db->sql_query_limit($sql, $row_limit);
			while($row = $db->sql_fetchrow($result))
			{
				$comment_track_cache[$row['comment_id']] = false;
				$row_time = $row['time'];

				$wall_row[$row_time] = array(
					'id'			=> 'comment-' . $row['comment_id'],
					'user_id'		=> $row['user_id'],
					'parent_id'		=> !empty($row['parent_id']),

					'text'			=> $row['text'],
					'time'			=> $row_time,
				);

				$user_cache[$row['user_id']] = $row['user_id'];
			}
			$db->sql_freeresult($result);
		}
		
		$template->assign_vars(array(
			'S_SEARCH_TITLE'	=> $user->lang('RESULTS_OF', $keywords),
			'S_KEYWORDS'		=> $keywords,
		));
	}
	
	function _search_users($keywords, $main_search = false)
	{
		global $db, $user, $template, $follower;
		global $row_limit, $start, $user_cache, $wall_row, $show_next;
		
		$sql_ary = array(
			'SELECT'	=> 'u.*',
			'FROM'		=> array(USERS_TABLE => 'u'),
			'WHERE'		=> '(
				u.username_clean ' . $db->sql_like_expression($db->any_char . utf8_clean_string($keywords) . $db->any_char) . '
				OR LOWER(u.user_follow_name) ' .  $db->sql_like_expression($db->any_char . strtolower($keywords) . $db->any_char) . '
			)
			AND u.user_type <> ' . USER_IGNORE,
			'ORDER_BY'	=> 'username_clean ASC'
		);
		
		if($main_search)
		{
			// Count for pagination
			$count_sql = $db->sql_build_query('SELECT', array(
				'SELECT'	=> 'COUNT(*) as count',
				'FROM'		=> $sql_ary['FROM'],
				'WHERE'		=> $sql_ary['WHERE']
			));
			$result = $db->sql_query($count_sql);
			$row_count = $db->sql_fetchfield('count');
			$db->sql_freeresult($result);
			
			$sql = $db->sql_build_query('SELECT', $sql_ary);
			$result = $db->sql_query_limit($sql, $row_limit, $start);
			while($row = $db->sql_fetchrow($result))
			{
				$user_id = (int)$row['user_id'];

				$wall_row[$user_id] = array(
					'id'				=> 'user-' . $user_id,
					'user_id'			=> $user_id,
				);

				$user_cache[$user_id] = $user_id;
			}
			$db->sql_freeresult($result);
			
			if(!empty($wall_row))
			{
				// Check if I following this users
				$sql = 'SELECT follow_id
					FROM ' . FOLLOWER_TRACK_TABLE . '
					WHERE user_id = ' . $user->data['user_id'] . '
					AND ' . $db->sql_in_set('follow_id', $user_cache) . '
					AND follower = 1';
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$wall_row[$row['follow_id']]['is_following'] = true;
				}
				$db->sql_freeresult($result);

				$template->assign_var('PAGINATION', generate_pagination($follower->u_action . 'mode=search&amp;type=user&amp;keywords=' . urlencode($keywords), $row_count, $row_limit, $start));
			}
			
			$template->assign_vars(array(
				'S_SEARCH_TITLE'		=> $user->lang('RESULTS_USER', $keywords),
				'S_KEYWORDS'			=> $keywords,
				'U_VIEW_ALL_COMMENTS'	=> $follower->u_action . 'mode=search&amp;keywords=' . urlencode($keywords)
			));

			$show_next = false;
		}
		else
		{
			$sql = $db->sql_build_query('SELECT', $sql_ary);
			$result = $db->sql_query_limit($sql, 5);
			while($row = $db->sql_fetchrow($result))
			{
				$template->assign_block_vars('user_search', array(
					'S_AVATAR'			=> ($user->optionget('viewavatars')) ? get_user_avatar($row['user_avatar'], $row['user_avatar_type'], $row['user_avatar_width'], $row['user_avatar_height']) : '',
					'S_FOLLOWNAME'		=> get_followname_string('label', $row['user_follow_name'], $row['user_id']),
					'S_FULLNAME'		=> $row['username'],
					
					'U_FOLLOWNAME'		=> get_followname_string('profile', $row['user_follow_name'], $row['user_id']),
				));
			}
			$db->sql_freeresult($result);
			
			$template->assign_var('U_VIEW_ALL_USERS', $follower->u_action . 'mode=search&amp;type=user&amp;keywords=' . urlencode($keywords));
		}
		
	}
}
?>
