<?php
/*
*
* @name schema_install.php
* @package cBB Follower Installer
* @version $Id: schema_install.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

if(!defined('IN_PHPBB') || !defined('IN_INSTALL'))
{
	exit;
}

$this->schema = array(
	/* Table definitions */
	'table_add' => array(
		FOLLOWER_COMMENTS_TABLE		=> array(
			'COLUMNS' => array(
				'comment_id'	=> array('UINT:11', NULL, 'auto_increment'),
				'parent_id'		=> array('UINT:11', 0),
				'user_id'		=> array('UINT:8', 0),
				'comment_text'	=> array('TEXT', ''),
				'comment_time'	=> array('UINT:11', 0),
			),
			'PRIMARY_KEY'	=> 'comment_id',
		),
		
		FOLLOWER_HASHTAGS_TABLE		=> array(
			'COLUMNS' => array(
				'hashtag'		=> array('VCHAR', ''),
				'comment_id'	=> array('UINT:11', 0),
				'post_id'		=> array('UINT:8', 0),
			),
			'KEYS'			=> array(
				'hashtag'		=> array('INDEX', 'hashtag'),
			),
		),

		FOLLOWER_MENTIONS_TABLE		=> array(
			'COLUMNS' => array(
				'user_id'		=> array('UINT:8', 0),
				'comment_id'	=> array('UINT:11', 0),
				'post_id'		=> array('UINT:8', 0),
				'mention_time'	=> array('UINT:11', 0),
			),
			'KEYS'			=> array(
				'user_id'		=> array('INDEX', 'user_id'),
			),
		),

		FOLLOWER_TRACK_TABLE	=> array(
			'COLUMNS' => array(
				'user_id'		=> array('UINT:8', 0),
				'follow_id'		=> array('UINT:8', 0),
				'comment_id'	=> array('UINT:11', 0),
				'follower'		=> array('BOOL', 0),
				'share'			=> array('BOOL', 0),
				'track_time'	=> array('UINT:11', 0),
			),
			'KEYS'			=> array(
				'user_id'		=> array('INDEX', 'user_id'),
			),
		),
	),
	
	/* Add a new columns to users table */
	'table_column_add' => array(
		array(USERS_TABLE, 'user_follow_name',		array('VCHAR:255', '')),
		array(USERS_TABLE, 'user_followers',		array('UINT:8', 0)),
		array(USERS_TABLE, 'user_following',		array('UINT:8', 0)),
		array(USERS_TABLE, 'user_comments',			array('UINT:8', 0)),
		array(USERS_TABLE, 'user_aboutme',			array('MTEXT', '')),
		array(USERS_TABLE, 'user_notify_options',	array('UINT:11', 0)),
		array(USERS_TABLE, 'user_new_notify',		array('UINT:4', 0)),
	),
	
	// Lets add a config settings
	'config_add' => array(
		array('fwr_enabled', true),
		array('fwr_default', false),
		array('fwr_allow_notify', FWR_NOTIFY_ALL),
		array('fwr_allow_comments', true),
		array('fwr_row_limit', 25),
		array('fwr_maxlength', 140),
		array('fwr_store_time', 0),
		array('fwr_slider_autohide', false),
		array('fwr_slider_pos', 'center'),
		array('fwr_version', $this->new_version),
	),
	
	// Define modules to install
	'module_check' => 0, // $this->schema['modules'] array ID to check or false to disable

	'modules' => array(
		// Add the ACP modules
		array('acp', 'ACP_CAT_DOT_MODS', 'ACP_FOLLOWER'),
		array('acp', 'ACP_FOLLOWER', array(
			'module_basename'	=> 'follower',
			'modes'				=> array('config', 'user', 'tools'),
		)),
		
		// Add the UCP modules
		array('ucp', 0, 'UCP_FOLLOWER'),
		array('ucp', 'UCP_FOLLOWER', array(
			'module_basename'	=> 'follower',
			'modes'				=> array('profile', 'notify'),
		)),
	),

	// Define permissions to add and set
	'permission_add' => array(
		array('u_fwr_publish',	true),
		array('u_fwr_share',	true),
		array('u_fwr_delete',	true),
		array('u_fwr_follow',	true),
		array('u_fwr_mention',	true),

		array('m_fwr_delete',	true),
	),

	'permission_set' => array(
		array('REGISTERED',		'u_fwr_publish', 'group'),
		array('REGISTERED',		'u_fwr_share', 'group'),
		array('ROLE_USER_FULL',	'u_fwr_delete'),
		array('REGISTERED',		'u_fwr_follow', 'group'),
		array('REGISTERED',		'u_fwr_mention', 'group'),
		
		array('ROLE_MOD_FULL',	'm_fwr_delete'),
	)
);

function sync_userdata()
{
	global $db;

	// Delete the spaces in the usernames to use as follow names and set default notify options
	$sql = 'UPDATE ' . USERS_TABLE . "
		SET user_follow_name = REPLACE(username, ' ', ''),
			user_notify_options = 2600
		WHERE user_type <> " . USER_IGNORE;
	$db->sql_query($sql);
	
	return true;
}
?>
