<?php
/*
*
* @name acp_follower.php
* @package phpBB3 cBB Follower
* @version $Id: acp_follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_follower
{
	var $u_action;
	var $new_config = array();

	function main($id, $mode)
	{
		global $db, $cache, $user, $template, $config, $phpbb_root_path, $phpEx;

		$submit = (isset($_POST['submit']) ? true : false);
		$error	= array();

		switch($mode)
		{
			case 'config':
				$display_vars = array(
					'title'	=> 'ACP_FOLLOWER',
					'vars'	=> array(
						'legend1'				=> 'ACP_FOLLOWER_CONFIG',
						'fwr_enabled'			=> array('lang' => 'FWR_ENABLED',			'validate' => 'bool',		'type' => 'radio:yes_no'),
						'fwr_default'			=> array('lang' => 'FWR_DEFAULT',			'validate' => 'bool',		'type' => 'radio:yes_no'),
						'fwr_row_limit'			=> array('lang' => 'FWR_ROW_LIMIT',			'validate' => 'int:1:100',	'type' => 'text:4:3'),
						'fwr_allow_notify'		=> array('lang' => 'FWR_NOTIFY',			'validate' => 'int',		'type' => 'select',	'method' => 'make_select',	'params' => array('notify', '{CONFIG_VALUE}')),

						'legend2'				=> 'ACP_FOLLOWER_COMMENTS',
						'fwr_allow_comments'	=> array('lang' => 'FWR_ALLOW_COMMENTS',	'validate' => 'bool',		'type' => 'radio:yes_no'),
						'fwr_maxlength'			=> array('lang' => 'FWR_MAXLENGTH',			'validate' => 'int:1:200',	'type' => 'text:4:3'),
						'fwr_store_time'		=> array('lang' => 'FWR_STORE_TIME',		'validate' => 'int',		'type' => 'select',	'method' => 'make_select',	'params' => array('store_time', '{CONFIG_VALUE}')),

						'legend3'				=> 'ACP_FOLLOWER_SLIDER',
						'fwr_slider_autohide'	=> array('lang' => 'FWR_SLIDER_AUTOHIDE',	'validate' => 'bool',		'type' => 'radio:yes_no'),
						'fwr_slider_pos'		=> array('lang' => 'FWR_SLIDER_POS',		'validate' => 'string',		'type' => 'select',	'method' => 'make_select',	'params' => array('slider_pos', '{CONFIG_VALUE}')),

						'legend4'				=> 'ACP_SUBMIT_CHANGES',
					)
				);
				
				$this->new_config = $config;
				$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;

				// We validate the complete config if whished
				validate_config_vars($display_vars['vars'], $cfg_array, $error);

				if ($submit && !check_form_key('acp_follower'))
				{
					$error[] = $user->lang['FORM_INVALID'];
				}

				// Do not write values if there is an error
				if (sizeof($error))
				{
					$submit = false;
				}

				// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
				foreach ($display_vars['vars'] as $config_name => $null)
				{
					if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
					{
						continue;
					}

					$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

					if ($submit)
					{
						set_config($config_name, $config_value);
					}
				}

				if($submit)
				{
					// Enable/Disable the ucp modules
					$sql = 'UPDATE ' . MODULES_TABLE . '
						SET module_enabled = ' . (int)$this->new_config['fwr_enabled'] . "
						WHERE module_class = 'ucp'
						AND module_basename = 'follower'";
					$db->sql_query($sql);
					
					// If changed the status destroy the cache
					if($db->sql_affectedrows())
					{
						$cache->destroy('_modules_ucp');
						$cache->destroy('sql', MODULES_TABLE);
					}
					
					add_log('admin', 'LOG_CONFIG_FWR');
					trigger_error($user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
				}
				
				// Check version
				$this->check_version_info(request_var('versioncheck_force', false));
			break;
			
			case 'user':
				$user->add_lang(array('acp/users', 'mods/info_ucp_follower'));

				$display_vars = array(
					'title'	=> 'USER_ADMIN',
					'vars'	=> array(
						'legend1'				=> 'USER_PROFILE',
						'user_follow_name'		=> array('lang' => 'FOLLOW_NAME',		'validate' => 'string',		'type' => 'text:40:255'),
						'user_aboutme'			=> array('lang' => 'ABOUTME',			'validate' => 'string',		'type' => 'textarea:5:5'),

						'legend2'				=> 'ACP_SUBMIT_CHANGES',
					)
				);
				
				$username	= utf8_normalize_nfc(request_var('username', '', true));
				$user_id	= request_var('u', 0);
				
				if(!$username && !$user_id)
				{
					$this->page_title = 'SELECT_USER';

					$template->assign_vars(array(
						'S_SELECT_USER'		=> true,
						'U_FIND_USERNAME'	=> append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&amp;form=acp_follower&amp;field=username&amp;select_single=true'),
					));
					
					break;
				}
				
				if($user_id)
				{
					$sql_where = "user_id = $user_id";
				}
				else
				{
					$sql_where = "username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'
						OR user_follow_name = '" . $db->sql_escape($username) . "'";
				}
				
				$sql = 'SELECT user_id, username, user_follow_name, user_aboutme
					FROM ' . USERS_TABLE . "
					WHERE $sql_where";
				$result = $db->sql_query($sql);
				$user_row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if(!$user_row)
				{
					trigger_error($user->lang['NO_USER'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				
				if(!$user_id)
				{
					$user_id = (int)$user_row['user_id'];
				}
				unset($user_row['user_id']);
				
				$this->new_config = $user_row;
				$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;

				// We validate the complete config if whished
				validate_config_vars($display_vars['vars'], $cfg_array, $error);

				if($submit && !check_form_key('acp_follower'))
				{
					$error[] = $user->lang['FORM_INVALID'];
				}

				// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
				if($submit)
				{
					foreach ($display_vars['vars'] as $config_name => $null)
					{
						if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
						{
							continue;
						}

						$this->new_config[$config_name] = $cfg_array[$config_name];
					}
					
					if(!preg_match('#^([^\s@<>:]+)$#', $this->new_config['user_follow_name']))
					{
						$error[] = $user->lang['WRONG_DATA_FOLLOW_NAME'];
					}
					
					// Check if the followname already exists
					if($this->new_config['user_follow_name'] != $user_row['user_follow_name'])
					{
						$sql = 'SELECT user_follow_name
							FROM ' . USERS_TABLE . "
							WHERE user_follow_name = '" . $db->sql_escape($this->new_config['user_follow_name']) . "'";
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if($row)
						{
							$error[] = 'FOLLOW_NAME_TAKEN';
						}
					}
				}
				
				// Do not write values if there is an error
				if(sizeof($error))
				{
					$submit = false;
				}
				
				if($submit)
				{
					$sql = 'UPDATE ' . USERS_TABLE . '
						SET ' . $db->sql_build_array('UPDATE', $this->new_config) . '
						WHERE user_id = ' . $user_id;
					$db->sql_query($sql);
					
					add_log('admin', 'LOG_USER_USER_UPDATE', $user_row['username']);
					trigger_error($user->lang['USER_PROFILE_UPDATED'] . adm_back_link($this->u_action));
				}
				
				$this->u_action .= '&amp;u=' . $user_id;
			break;
			
			case 'tools':
				$action = request_var('action', '');

				$display_vars = array(
					'title'	=> 'ACP_FOLLOWER_TOOLS',
					'vars'	=> array()
				);
				
				switch($action)
				{
					case 'stats':
						$comments_count = $followers_count = $following_count = array();
						
						// Select comments count
						$sql = 'SELECT COUNT(comment_id) as stat, user_id
							FROM ' . FOLLOWER_COMMENTS_TABLE . '
							GROUP BY user_id';
						$result = $db->sql_query($sql);
						while($row = $db->sql_fetchrow($result))
						{
							$comments_count[$row['user_id']] = $row['stat'];
						}
						$db->sql_freeresult($result);
						
						// Select followers count
						$sql = 'SELECT COUNT(user_id) AS stat, follow_id
							FROM ' . FOLLOWER_TRACK_TABLE . '
							GROUP BY follow_id';
						$result = $db->sql_query($sql);
						while($row = $db->sql_fetchrow($result))
						{
							$followers_count[$row['follow_id']] = $row['stat'];
						}
						$db->sql_freeresult($result);
						
						// Select following count
						$sql = 'SELECT COUNT(follow_id) AS stat, user_id
							FROM ' . FOLLOWER_TRACK_TABLE . '
							GROUP BY user_id';
						$result = $db->sql_query($sql);
						while($row = $db->sql_fetchrow($result))
						{
							$following_count[$row['user_id']] = $row['stat'];
						}
						$db->sql_freeresult($result);
						
						// Reset the values to all users
						$sql = 'UPDATE ' . USERS_TABLE . '
							SET user_comments = 0, user_followers = 0, user_following = 0';
						$db->sql_query($sql);

						// Update the values
						if(sizeof($comments_count))
						{
							foreach($comments_count as $row_user_id => $count)
							{
								$sql = 'UPDATE ' . USERS_TABLE . "
									SET user_comments = $count
									WHERE user_id = $row_user_id";
								$db->sql_query($sql);
							}
						}
						
						if(sizeof($followers_count))
						{
							foreach($followers_count as $row_user_id => $count)
							{
								$sql = 'UPDATE ' . USERS_TABLE . "
									SET user_followers = $count
									WHERE user_id = $row_user_id";
								$db->sql_query($sql);
							}
						}
						
						if(sizeof($following_count))
						{
							foreach($following_count as $row_user_id => $count)
							{
								$sql = 'UPDATE ' . USERS_TABLE . "
									SET user_following = $count
									WHERE user_id = $row_user_id";
								$db->sql_query($sql);
							}
						}
					break;
				}
				
				$template->assign_var('TOOLS_MODE', true);
			break;
			
			default:
				trigger_error('MODULE_NOT_ACCESIBLE', E_USER_ERROR);
			break;
		}

		add_form_key('acp_follower');

		$this->tpl_name	= 'acp_follower';
		
		if(empty($this->page_title))
		{
			$this->page_title	= $display_vars['title'];
		}

		$template->assign_vars(array(
			'L_TITLE'			=> $user->lang[$display_vars['title']],
			'L_TITLE_EXPLAIN'	=> $user->lang[$display_vars['title'] . '_EXPLAIN'],

			'S_ERROR'		=> (sizeof($error) ? implode('<br />', $error) : ''),
			'U_ACTION'		=> $this->u_action
		));

		// Output relevant page
		foreach ($display_vars['vars'] as $config_key => $vars)
		{
			if (!is_array($vars) && strpos($config_key, 'legend') === false)
			{
				continue;
			}

			if (strpos($config_key, 'legend') !== false)
			{
				$template->assign_block_vars('options', array(
					'S_LEGEND'		=> true,
					'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars)
				);

				continue;
			}

			$type = explode(':', $vars['type']);

			$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

			if (empty($content))
			{
				continue;
			}

			$template->assign_block_vars('options', array(
				'KEY'			=> $config_key,
				'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
				'EXPLAIN'		=> (isset($user->lang[$vars['lang'] . '_EXPLAIN']) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : ''),
				'CONTENT'		=> $content,
			));

			unset($display_vars['vars'][$config_key]);
		}
	}

	/**
	 * make_select
	 * Build select input for config
	*/
	function make_select($type, $select = false)
	{
		global $user;
	
		$string = '';
		
		switch($type)
		{
			case 'notify':
				$ary = array(
					'NOTIFY_DISABLED'		=> FWR_NOTIFY_NONE,
					'NOTIFY_ONLY_MP'		=> FWR_NOTIFY_MP,
					'NOTIFY_ONLY_EMAIL'		=> FWR_NOTIFY_EMAIL,
					'NOTIFY_ALL'			=> FWR_NOTIFY_ALL,
				);
			break;
			
			case 'store_time':
				$ary = array(
					'NO_LIMIT'		=> 0,
					'ONE_DAY'		=> 86400,
					'ONE_WEEK'		=> 604800,
					'ONE_MONTH'		=> 2592000,
					'ONE_YEAR'		=> 31104000,
					'TWO_YEARS'		=> 62208000,
				);
			break;
			
			case 'slider_pos':
				$ary = array(
					'LEFT_POS'		=> 'left',
					'CENTER_POS'	=> 'center',
					'RIGHT_POS'		=> 'right',
				);
			break;
			
			default:
				return '';
			break;	
		}
	
		foreach($ary as $lang => $value)
		{
			$selected	= (($value == $select) ? ' selected="selected"' : '');
			$title		= (isset($user->lang[$lang]) ? $user->lang[$lang] : $lang);
	
			$string .= '<option value="' . $value . '"' . $selected . '>' . $title . '</option>';
		}
	
		return $string;
	}

	/**
	 * Check the version information of cBB Follower
	 *
	 * @param bool $force_update Ignores cached data. Defaults to false.
	 * @param int $ttl Cache version information for $ttl seconds. Defaults to 259200 (72 hours).
	 *
	 */
	function check_version_info($force_update = false, $ttl = 259200)
	{
		global $config, $template, $cache;

		$info = $cache->get('fwr_versioncheck');

		if ($info === false || $force_update)
		{
			$errstr = '';
			$errno = 0;

			$info = get_remote_file('canidev.com', '/app/info', 'cbb-follower-10x.txt', $errstr, $errno);

			if ($info === false)
			{
				$cache->destroy('fwr_versioncheck');
				return false;
			}

			$cache->put('fwr_versioncheck', $info, $ttl);
		}

		$current_version = (isset($config['fwr_version']) ? $config['fwr_version'] : '1.0.0');

		$latest_version_info = ($info) ? explode("\n", $info, 2) : array($current_version, '#');
		$latest_version = trim($latest_version_info[0]);

		$template->assign_vars(array(
			'S_VERSION_NOT_UP_TO_DATE'	=> version_compare($current_version, $latest_version, '<') ? true : false,

			'U_VERSIONCHECK_FORCE'	=> $this->u_action . '&amp;versioncheck_force=1',
			'U_MORE_INFORMATION'	=> trim($latest_version_info[1]),
		));
	}
}

?>
