<?php
/*
*
* @name hook_follower.php
* @package phpBB3 cBB Follower
* @version $Id: hook_follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

require($phpbb_root_path . 'follower/includes/follower.' . $phpEx);
$follower = new follower();

/* Register the hook to execute on user->setup() */
$phpbb_hook->register('phpbb_user_session_handler', array($follower, 'setup'));

class follower_hook
{
	static public function bbcode_nl2br(&$text)
	{
		global $follower;

		$follower->display($text);
	}

	static public function delete_post(&$table_ary)
	{
		if(self::check())
		{
			$table_ary = array_merge($table_ary, array(FOLLOWER_MENTIONS_TABLE, FOLLOWER_HASHTAGS_TABLE));
		}
	}
	
	static public function get_username_string(&$_profile_cache)
	{
		global $follower;

		if(!empty($follower->default_profile))
		{
			$_profile_cache['base_url'] = $follower->u_action . 'u={USER_ID}&amp;mode=wall';
		}
	}
	
	static public function memberlist($member_id)
	{
		global $db, $user, $template, $follower;

		if($follower->active)
		{
			$sql = 'SELECT follower
				FROM ' . FOLLOWER_TRACK_TABLE . "
				WHERE follow_id = $member_id
				AND follower = 1
				AND user_id = " . $user->data['user_id'];
			$result = $db->sql_query($sql);
			$follow = (int)$db->sql_fetchfield('follower');
			$db->sql_freeresult($result);
			
			$button_params = array(
				'USER_ID'	=> $member_id,
				'USER_STATUS'	=> ($follow) ? 'following' : 'follow',
			);
			
			$template->assign_vars(array(
				'S_FOLLOW_BUTTON'	=> ($follower->can_follow && $member_id != $user->data['user_id']) ? $follower->buttons->create('follow_button', $button_params) : '',
				'S_WALL_BUTTON'		=> $follower->buttons->create('wall_button', array(
					'ACTION'	=> $follower->u_action . 'mode=wall&amp;u=' . $member_id
				))
			));
		}
	}
	
	static public function submit_post($data, $mode, $current_time)
	{
		global $follower;
		
		if ($data['post_id'] && in_array($mode, array('post', 'reply', 'quote', 'edit')))
		{
			$follower->store($data['post_id'], 0, $current_time);
		}
	}

	static public function topic_review(&$sql)
	{
		global $follower;

		if(!empty($follower->active))
		{
			$sql = str_replace('z.foe', 'z.foe, u.user_follow_name', $sql);
			return true;
		}
		
		return false;
	}
	
	static public function user_add(&$sql_ary)
	{
		global $db;

		if(!self::check())
		{
			return false;
		}
		
		$follow_name = str_replace(array('@', ' '), array('_', ''), $sql_ary['username']);
		
		/* Check if the follow name already exists */
		$sql = 'SELECT user_follow_name
			FROM ' . USERS_TABLE . "
			WHERE LOWER(user_follow_name) = '" . $db->sql_escape(strtolower($follow_name)) . "'";
		$result = $db->sql_query($sql);
		$user_follow_name = (string)$db->sql_fetchfield('user_follow_name');
		$result = $db->sql_freeresult($result);
		
		/* If exists, set new unique follow_name to the user */
		if($user_follow_name)
		{
			$follow_name .= '_' . gen_rand_string(4);
		}
		
		$sql_ary += array(
			'user_follow_name'	=> $follow_name,
			'user_aboutme'		=> '',
		);
		
		return true;
	}
	
	static public function user_delete(&$table_ary, $user_id)
	{
		global $db;

		if(self::check())
		{
			// Get followers and following to update counters
			$counter_ary = array(
				'followers'		=> array(),
				'following'		=> array(),
			);
			
			$sql = 'SELECT user_id, follow_id
				FROM ' . FOLLOWER_TRACK_TABLE . "
				WHERE (user_id = $user_id
				OR follow_id = $user_id)
				AND follower = 1";
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				if($row['user_id'] == $user_id)
				{
					$counter_ary['followers'][$row['follow_id']] = $row['follow_id'];
				}
				else
				{
					$counter_ary['following'][$row['user_id']] = $row['user_id'];
				}
			}
			$db->sql_freeresult($result);
			
			// Update followers counters
			if(sizeof($counter_ary['followers']))
			{
				$sql = 'UPDATE ' . USERS_TABLE . '
					SET user_followers = user_followers - 1
					WHERE user_followers > 0
					AND ' . $db->sql_in_set('user_id', $counter_ary['followers']);
				$db->sql_query($sql);
			}
			
			// Update following counters
			if(sizeof($counter_ary['following']))
			{
				$sql = 'UPDATE ' . USERS_TABLE . '
					SET user_following = user_following - 1
					WHERE user_following > 0
					AND ' . $db->sql_in_set('user_id', $counter_ary['following']);
				$db->sql_query($sql);
			}

			// Delete values in tables
			$table_ary = array_merge($table_ary, array(FOLLOWER_COMMENTS_TABLE, FOLLOWER_MENTIONS_TABLE, FOLLOWER_TRACK_TABLE));
		
			// Delete all the user followers
			$sql = 'DELETE FROM ' . FOLLOWER_TRACK_TABLE . "
				WHERE follow_id = $user_id";
			$db->sql_query($sql);
		}
	}
	
	static public function user_notification($topic_notification, $forum_id, $topic_id, $post_id, $sql_ignore_users)
	{
		global $phpbb_root_path, $phpEx;

		if(!function_exists('follower_notification'))
		{
			include($phpbb_root_path . 'follower/includes/functions_follower.' . $phpEx);
		}

		follower_notification((($topic_notification) ? 'posts' : 'topics'), $post_id, array('forum_id' => $forum_id, 'topic_id' => $topic_id), $sql_ignore_users);
	}
	
	static public function viewtopic(&$postrow, $poster_id)
	{
		global $user_cache, $db, $user, $follower;
		static $fwr_user_cache;

		if($poster_id == ANONYMOUS || empty($user_cache) || empty($follower->active))
		{
			return false;
		}
		
		if(empty($fwr_user_cache))
		{
			$fwr_user_cache = array();

			$sql = 'SELECT u.user_id, u.user_follow_name, ft.follower
				FROM ' . USERS_TABLE . ' u
				LEFT JOIN ' . FOLLOWER_TRACK_TABLE . ' ft
					ON(ft.user_id = ' . $user->data['user_id'] . '
					AND ft.follow_id = u.user_id
					AND ft.follower = 1)
				WHERE ' . $db->sql_in_set('u.user_id', array_keys($user_cache));
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$fwr_user_cache[$row['user_id']] = $row;
			}
		}

		if(isset($fwr_user_cache[$poster_id]))
		{
			$button_params = array(
				'USER_ID'		=> $poster_id,
				'USER_STATUS'	=> ($fwr_user_cache[$poster_id]['follower']) ? 'following' : 'follow',
			);
			
			$postrow += array(
				'AUTHOR_FOLLOW_NAME'	=> get_followname_string('name_active', $fwr_user_cache[$poster_id]['user_follow_name']),
				'S_FOLLOW_BUTTON'		=> ($follower->can_follow && $poster_id != $user->data['user_id']) ? $follower->buttons->create('follow_button', $button_params) : '',
			);
		}
	}
	
	static public function create_button($tpl_name, $values)
	{
		global $follower;
	
		if(!empty($follower->active))
		{
			return $follower->buttons->create($tpl_name, $values);
		}
		
		return '';
	}
	
	static private function check()
	{
		global $follower;
	
		if(!empty($follower->installed))
		{
			return true;
		}
		
		return false;
	}
}

?>