<?php
/*
*
* @name ucp_follower.php
* @package ucp cBB Follower
* @version $Id: ucp_follower.php,v1.0.1 01/04/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* ucp_follower
* Changing follower profile settings
*
* @package ucp
*/
class ucp_follower
{
	var $u_action;

	function main($id, $mode)
	{
		global $db, $user, $config, $template, $phpbb_root_path;
		global $follower;
		
		if(!$follower->active)
		{
			trigger_error('NOT_AUTHORISED');
		}

		$submit	= (!empty($_POST['submit'])) ? true : false;

		$error = $data = array();
		$s_hidden_fields = '';

		add_form_key('ucp_follower');

		switch ($mode)
		{
			case 'profile':
				$data = array(
					'follow_name'	=> utf8_normalize_nfc(request_var('follow_name', $user->data['user_follow_name'], true)),
					'aboutme'		=> utf8_normalize_nfc(request_var('aboutme', $user->data['user_aboutme'], true)),
				);

				if ($submit)
				{
					// Do not check cur_password, it is the old one.
					$check_ary = array(
						'follow_name'		=> array('match', false, '#^([^\s@<>:]+)$#'),
						'aboutme'			=> array('string', true, 0, 160),
					);

					$error = validate_data($data, $check_ary);

					if(!check_form_key('ucp_follower'))
					{
						$error[] = 'FORM_INVALID';
					}

					// Check if the followname already exists
					if($data['follow_name'] != $user->data['user_follow_name'])
					{
						$sql = 'SELECT user_follow_name
							FROM ' . USERS_TABLE . "
							WHERE LOWER(user_follow_name) = '" . $db->sql_escape(strtolower($data['follow_name'])) . "'";
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if($row)
						{
							$error[] = 'FOLLOW_NAME_TAKEN';
						}
					}

					if(!sizeof($error))
					{
						$sql_ary = array(
							'user_follow_name'	=> $data['follow_name'],
							'user_aboutme'		=> $data['aboutme'],
						);

						$sql = 'UPDATE ' . USERS_TABLE . '
							SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
							WHERE user_id = ' . $user->data['user_id'];
						$db->sql_query($sql);

						meta_refresh(3, $this->u_action);
						$message = $user->lang['PROFILE_UPDATED'] . '<br /><br />' . sprintf($user->lang['RETURN_UCP'], '<a href="' . $this->u_action . '">', '</a>');

						trigger_error($message);
					}

					// Replace "error" strings with their real, localised form
					$error = preg_replace('#^([A-Z_]+)$#e', "(!empty(\$user->lang['\\1'])) ? \$user->lang['\\1'] : '\\1'", $error);
				}

				$template->assign_vars(array(
					'S_FOLLOW_NAME'		=> $data['follow_name'],
					'S_ABOUTME'			=> $data['aboutme'],
				));
			break;

			case 'notify':
				$display_vars = array();
				
				if($config['fwr_allow_comments'])
				{
					$display_vars += array(
						'legend1'		=> 'COMMENT_NOTIFY',
						'replies'		=> 'NOTIFY_REPLIES',
						'mentions'		=> 'NOTIFY_MENTIONS'
					);
				}

				$display_vars += array(
					'legend2'		=> 'ACTIVITY_NOTIFY',
					'follower'		=> 'NOTIFY_FOLLOWER',
					'share'			=> 'NOTIFY_SHARE',
					'topics'		=> 'NOTIFY_TOPICS',
					'posts'			=> 'NOTIFY_POSTS',
				);

				if($submit)
				{
					if(check_form_key('ucp_follower'))
					{
						$data = request_var('notify', array('' => 0));

						foreach($display_vars as $key => $params)
						{
							if(strpos($key, 'legend') !== false)
							{
								continue;
							}

							$follower->notify($key, 'pm', false, (($data[$key] == 1) ? 1 : 0));
							$follower->notify($key, 'email', false, (($data[$key] == 2) ? 1 : 0));
						}

						$sql = 'UPDATE ' . USERS_TABLE . "
							SET user_notify_options = '" . $user->data['user_notify_options'] . "'
							WHERE user_id = " . $user->data['user_id'];
						$db->sql_query($sql);

						meta_refresh(3, $this->u_action);
						$message = $user->lang['PREFERENCES_UPDATED'] . '<br /><br />' . sprintf($user->lang['RETURN_UCP'], '<a href="' . $this->u_action . '">', '</a>');
						trigger_error($message);
					}
				}

				foreach($display_vars as $key => $lang_key)
				{
					$var = array();

					$s_title = (isset($user->lang[$lang_key]) ? $user->lang[$lang_key] : $lang_key);

					if(strpos($key, 'legend') !== false)
					{
						$var = array(
							'IS_LEGEND'	=> true,
							'S_TITLE'	=> $s_title,
						);
					}
					else
					{
						$disabled_pm		= ($config['fwr_allow_notify'] != FWR_NOTIFY_MP && $config['fwr_allow_notify'] != FWR_NOTIFY_ALL) ? true : false;
						$disabled_email		= ($config['fwr_allow_notify'] != FWR_NOTIFY_EMAIL && $config['fwr_allow_notify'] != FWR_NOTIFY_ALL) ? true : false;

						$selected_pm		= ($disabled_pm) ? ' disabled="disabled"' : ($follower->notify($key, 'pm') ? ' selected="selected"' : '');
						$selected_email		= ($disabled_email) ? ' disabled="disabled"' : ($follower->notify($key, 'email') ? ' selected="selected"' : '');

						$s_options = '<select name="notify[' . $key . ']">
							<option value="' . FWR_NOTIFY_NONE . '">' . $user->lang['NOTIFY_NONE'] . '</option>
							<option value="' . FWR_NOTIFY_MP . '"' . $selected_pm . '>' . $user->lang['NOTIFY_PM'] . '</option>
							<option value="' . FWR_NOTIFY_EMAIL . '"' . $selected_email . '>' . $user->lang['NOTIFY_EMAIL'] . '</option>
						</select>';

						$var = array(
							'S_TITLE'		=> '<label>' . $s_title . '</label>',
							'S_OPTIONS'		=> $s_options,
						);
					}

					$template->assign_block_vars('notify_row', $var);
				}
			break;
		}

		$template->assign_vars(array(
			'L_TITLE'	=> $user->lang['UCP_FOLLOWER_' . strtoupper($mode)],
			'S_MODE'	=> $mode,
			'ERROR'		=> (sizeof($error)) ? implode('<br />', $error) : '',

			'S_UCP_ACTION'		=> $this->u_action,
		));

		// Set desired template
		$this->tpl_name = 'follower/ucp_follower';
		$this->page_title = 'UCP_FOLLOWER_' . strtoupper($mode);
	}
}

?>
